 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.nio.charset.Charset;

/*
 * This class provides the "Apply lexical resources" window
 *
 */
public class ApplyLexicalResourcesFrame extends JInternalFrame implements InternalFrameListener{

	public static ApplyLexicalResourcesFrame frame;
	public JPanel main=new JPanel();
	public JPanel dicListPanel=new JPanel();
	public JPanel threeButtonsPanel=new JPanel();
	public JButton goButton=new JButton("Apply Selected Resources");
	public JList userDicList;
	public JList systemDicList;
	

	public ApplyLexicalResourcesFrame() {
		super("Lexical Resources",false,true);
		constructMainPanel();
		setContentPane(main);
		pack();
		setVisible(false);
		addInternalFrameListener(this);
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	}

		
	public static void init() {
		frame=new ApplyLexicalResourcesFrame();
	}


	public static ApplyLexicalResourcesFrame getFrame() {
		return frame;
	}


	public static void showFrame() {
		if (frame.isVisible()) {
			return;
		}
		frame.refreshDicLists();
		frame.setVisible(true);
		try { 
		    frame.setSelected(true);
		    frame.setIcon(false);
		} catch (java.beans.PropertyVetoException e2) {}
		
	}
	
	
	public void constructMainPanel() {
		main.setOpaque(true);
		main.setLayout(new BorderLayout());
		constructDicListPanel();
		constructThreeButtonsPanel();
		constructGoButton();
		main.add(dicListPanel,BorderLayout.CENTER);
		JPanel buttonsPanel=new JPanel();
		buttonsPanel.setLayout(new GridLayout(2,1));
		buttonsPanel.add(threeButtonsPanel);
		buttonsPanel.add(goButton);
		main.add(buttonsPanel,BorderLayout.SOUTH);
	}
	
	
	public void refreshDicLists() {
		Vector userListDef=loadDefaultDicList(Config.getUserCurrentLanguageDir()+"user_dic.def");
		Vector systemListDef=loadDefaultDicList(Config.getUserCurrentLanguageDir()+"system_dic.def");
		Vector userList=getDicList(Config.getUserCurrentLanguageDir()+"Dela"+File.separator);
		Vector systemList=getDicList(Config.getUnitexCurrentLanguageDir()+"Dela"+File.separator);
		userDicList.setListData(userList);
		systemDicList.setListData(systemList);
		userDicList.clearSelection();
		systemDicList.clearSelection();
		setDefaultSelection(userDicList,userListDef);
		setDefaultSelection(systemDicList,systemListDef);
	}
	
	
	public void constructDicListPanel() {
		dicListPanel.setLayout(new GridLayout(1,2));
		userDicList=new JList();
		systemDicList=new JList();
		userDicList.setBorder(BorderFactory.createLoweredBevelBorder());
		systemDicList.setBorder(BorderFactory.createLoweredBevelBorder());
		userDicList.setPreferredSize(new Dimension(200,400));
		systemDicList.setPreferredSize(new Dimension(200,400));
		JPanel userPanel=new JPanel();
		JPanel systemPanel=new JPanel();
		userPanel.setBorder(new TitledBorder("User resources"));
		systemPanel.setBorder(new TitledBorder("System resources"));
		JScrollPane scroll_1=new JScrollPane(userDicList);
		scroll_1.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scroll_1.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		JScrollPane scroll_2=new JScrollPane(systemDicList);
		scroll_2.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scroll_2.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		userPanel.add(scroll_1);
		systemPanel.add(scroll_2);
		dicListPanel.add(userPanel);
		dicListPanel.add(systemPanel);
	}


	public void constructThreeButtonsPanel() {
		threeButtonsPanel.setLayout(new GridLayout(1,3));
		JButton clearButton=new JButton("Clear");
		JButton defaultButton=new JButton("Default");
		JButton setDefaultButton=new JButton("Set Default");
		clearButton.addMouseListener(new ClearButtonListener());
		defaultButton.addMouseListener(new DefaultButtonListener());
		setDefaultButton.addMouseListener(new SetDefaultButtonListener());
		threeButtonsPanel.add(clearButton);
		threeButtonsPanel.add(defaultButton);
		threeButtonsPanel.add(setDefaultButton);
		threeButtonsPanel.setPreferredSize(new Dimension(300,40));
	}


	public void constructGoButton() {
		goButton.addMouseListener(new GoButtonListener());
	}


	public void internalFrameActivated(InternalFrameEvent e) {}
	public void internalFrameClosed(InternalFrameEvent e) {}
	public void internalFrameClosing(InternalFrameEvent e) {
		setVisible(false);
	}
	public void internalFrameDeactivated(InternalFrameEvent e) {}
	public void internalFrameDeiconified(InternalFrameEvent e) {}
	public void internalFrameIconified(InternalFrameEvent e) {}
	public void internalFrameOpened(InternalFrameEvent e) {}


class ClearButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run(){
				frame.userDicList.clearSelection();
				frame.systemDicList.clearSelection();
			}
		});
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class DefaultButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run(){
				frame.refreshDicLists();
			}
		});
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class SetDefaultButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run(){
				frame.saveDefaultDicLists();
			}
		});
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class GoButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		frame.setVisible(false);
		SwingUtilities.invokeLater(new Runnable() {
			public void run(){
				Object[] userSelection=userDicList.getSelectedValues();
				Object[] systemSelection=systemDicList.getSelectedValues();
				if ((userSelection==null || userSelection.length==0)
				    && (systemSelection==null || systemSelection.length==0)) {
				    	// if there is no dic selected, we do nothing
				    	return;
				}
				String[] command=new String[4];
				command[0]="\""+Config.getApplicationDir()+"Dico\"";
				command[0]=command[0]+" \""+Config.getCurrentSnt()+"\"";
				command[0]=command[0]+" \""+Config.getAlphabet()+"\"";
				if (userSelection!=null && userSelection.length!=0) {
					for (int i=0;i<userSelection.length;i++) {
						String param=" \""+Config.getUserCurrentLanguageDir();
						param=param+"Dela"+File.separator;
						param=param+((String)userSelection[i])+"\"";
						command[0]=command[0]+param;
					}
				}
				if (systemSelection!=null && systemSelection.length!=0) {
					for (int i=0;i<systemSelection.length;i++) {
						String param=" \""+Config.getUnitexCurrentLanguageDir();
						param=param+"Dela"+File.separator;
						param=param+((String)systemSelection[i])+"\"";
						command[0]=command[0]+param;
					}
				}
				// sorting DLF
				command[1]="\""+Config.getApplicationDir()+"SortTxt\"";
				command[1]=command[1]+" \""+Config.getCurrentSntDir()+"dlf\"";
				command[1]=command[1]+" -l \""+Config.getCurrentSntDir()+"dlf.n\"";
				if (Config.getCurrentLanguage().equals("Thai") ||
				   	Config.getCurrentLanguage().equals("Korean")) {
					command[1]=command[1]+" -thai ";
				} else {
					command[1]=command[1]+" -o \""+Config.getUserCurrentLanguageDir();
					command[1]=command[1]+"Alphabet_sort.txt\"";
				}
				// sorting DLC
				command[2]="\""+Config.getApplicationDir()+"SortTxt\"";
				command[2]=command[2]+" \""+Config.getCurrentSntDir()+"dlc\"";
				command[2]=command[2]+" -l \""+Config.getCurrentSntDir()+"dlc.n\"";
				if (Config.getCurrentLanguage().equals("Thai") ||
				   	Config.getCurrentLanguage().equals("Korean")) {
					command[2]=command[2]+" -thai ";
				} else {
					command[2]=command[2]+" -o \""+Config.getUserCurrentLanguageDir();
					command[2]=command[2]+"Alphabet_sort.txt\"";
				}
				// sorting ERR
				command[3]="\""+Config.getApplicationDir()+"SortTxt\"";
				command[3]=command[3]+" \""+Config.getCurrentSntDir()+"err\"";
				command[3]=command[3]+" -l \""+Config.getCurrentSntDir()+"err.n\"";
				if (Config.getCurrentLanguage().equals("Thai") ||
				   	Config.getCurrentLanguage().equals("Korean")) {
					command[3]=command[3]+" -thai ";
				} else {
					command[3]=command[3]+" -o \""+Config.getUserCurrentLanguageDir();
					command[3]=command[3]+"Alphabet_sort.txt\"";
				}
				TextDicFrame.close();
				new ProcessInfoFrame(command,true,new ApplyLexicalResourcesDo());
			}
		});
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


	public Vector getDicList(String name) {
		File F=new File(name);
		Vector v=new Vector();
		if (!F.exists()) return v;
		String files_list[]=F.list();
		for (int i=0;i<files_list.length;i++) {
			if (!(new File(name+files_list[i]).isDirectory()) && 
				(files_list[i].endsWith(".bin") || files_list[i].endsWith(".BIN"))) {
				v.add(files_list[i]);
			}
		}
		return v;
	}


	public Vector loadDefaultDicList(String name) {
		Vector v;
		BufferedReader br;
		try {
			br=new BufferedReader(new FileReader(name));
		} catch (FileNotFoundException e) {return null;}
		try {
			String s;
			v=new Vector();
			while ((s=br.readLine())!=null) {
				v.add(s);
			}
			br.close();
		} catch (IOException e) {return null;}
		return v;
	}


	public void setDefaultSelection(JList list,Vector v) {
		int[] indices=new int[100];
		int i=0;
		if (v==null) return;
		ListModel model=list.getModel();
		while (!v.isEmpty()) {
			String s=(String)(v.remove(0));
			int index=getElementIndex(model,s);
			if (index!=-1) {
				indices[i++]=index;
			}
		}
		if (i!=0) {
			int[] res=new int[i];
			for (int j=0;j<i;j++) {
				res[j]=indices[j];
			}
			list.setSelectedIndices(res);
		}
	}
	
	
	public int getElementIndex(ListModel model,String s) {
		if (model==null) return -1;
		int l=model.getSize();
		for (int i=0;i<l;i++) {
			if (s.equals((String)(model.getElementAt(i)))) {
				return i;
			}
		}
		return -1;
	}


	public void saveDefaultDicLists() {
		saveSelectionToFile(userDicList,Config.getUserCurrentLanguageDir()+"user_dic.def");
		saveSelectionToFile(systemDicList,Config.getUserCurrentLanguageDir()+"system_dic.def");
	}
	
	
	public void saveSelectionToFile(JList list,String name) {
		Object[] selection=list.getSelectedValues();
		File f=new File(name);
		try {
			if (!f.exists()) {
				f.createNewFile();
			}
			BufferedWriter bw=new BufferedWriter(new FileWriter(f));
			for (int i=0;i<selection.length;i++) {
				String s=((String)selection[i])+"\n";
				bw.write(s,0,s.length());
			}
			bw.close();
		} catch (IOException e) {}
	}

class ApplyLexicalResourcesDo extends DoAbstract {
	public void DO() {
		TextDicFrame.loadTextDic(Config.getCurrentSntDir());
		try {
			TextDicFrame.getFrame().setIcon(false);
			TextDicFrame.getFrame().setSelected(true);
		} catch (java.beans.PropertyVetoException e2) {}
	}	
}


}