 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import java.io.* ;

/*
 * This class describes the "Construct text FST" window
 *
 */
public class ConstructFstFrame extends JDialog {
	
	public JPanel panel=new JPanel();
	public JPanel normalizationPanel=new JPanel();
	public JPanel dicPanel=new JPanel();
	public JPanel buttons=new JPanel();
	public JButton OK=new JButton("Construct FST");
	public JButton CANCEL=new JButton("Cancel");
	public JCheckBox normFst=new JCheckBox("Apply the Normalization grammar (Norm.fst2)");
	public JCheckBox cleanFst=new JCheckBox("Clean Text FST");
	

	public ConstructFstFrame() {
		super(UnitexFrame.mainFrame,"Construct the Text FST",true);
		constructPanel();
		setContentPane(panel);
		pack();
		setResizable(false);
		setLocationRelativeTo(UnitexFrame.mainFrame);
		this.setVisible(true);
	}
	
	
	public void constructPanel() {
		panel.setLayout(new BorderLayout());
		constructNormalizationPanel();
		constructDicPanel();
		constructButtonsPanel();
		panel.add(normalizationPanel,BorderLayout.NORTH);
		panel.add(dicPanel,BorderLayout.CENTER);
		panel.add(buttons,BorderLayout.SOUTH);
	}
	
	
	public void constructNormalizationPanel() {
		normalizationPanel.setBorder(new TitledBorder("Normalization"));
		normFst.setSelected(true);
		cleanFst.setSelected(true);
		normalizationPanel.setLayout(new GridLayout(2,1));
		normalizationPanel.add(normFst);
		normalizationPanel.add(cleanFst);
	}


	public void constructDicPanel() {
		dicPanel.setBorder(new TitledBorder("Use Following Dictionaries previously constructed:"));
		dicPanel.setLayout(new GridLayout(3,1));
		dicPanel.add(new JLabel("The program will construct the text FST"));
		dicPanel.add(new JLabel("according to the DLF and DLC files previously"));
		dicPanel.add(new JLabel("constructed for the current text."));
	}
	

	public void constructButtonsPanel() {
		buttons.setBorder(new EmptyBorder(8,8,2,2));
		buttons.setLayout(new GridLayout(1,2));
		OK.addMouseListener(new OKListener());
		CANCEL.addMouseListener(new CANCELListener());
		buttons.add(CANCEL);
		buttons.add(OK);
	}



class OKListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		e.consume();
		setVisible(false);
		SwingUtilities.invokeLater(new Runnable() {
			public void run(){
				File dir=new File(Config.getCurrentSntDir());
				if (!dir.exists()) {
					// if the directory toto_snt does not exist, we create it
					dir.mkdir();
				}
				Config.deleteFileByName(Config.getCurrentSntDir()+"sentence*.grf");
				String command[]=new String[1];
				command[0]="\""+Config.getApplicationDir()+"Txt2Fst2\" \""+Config.getCurrentSnt()+"\"";
				command[0]=command[0]+" \""+Config.getAlphabet()+"\"";
				if (cleanFst.isSelected()) {
					command[0]=command[0]+" -clean";
				}
				if (normFst.isSelected() && new File(Config.getUserCurrentLanguageDir()+"Graphs"+File.separator+"Normalization"+File.separator+"Norm.fst2").exists()) {
					command[0]=command[0]+" \""+Config.getUserCurrentLanguageDir()+"Graphs";
					command[0]=command[0]+File.separator+"Normalization"+File.separator+"Norm.fst2\"";
				}
				TextAutomatonFrame.close();
				new ProcessInfoFrame(command,true,new ConstructFstDo());
			}
		});
		dispose();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class CANCELListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		setVisible(false);
		e.consume();
		dispose();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}



class ConstructFstDo extends DoAbstract {

	public ConstructFstDo() {
	}
	public void DO() {
		TextAutomatonFrame.showFrame();
		try {
			TextAutomatonFrame.getFrame().setIcon(false);
			TextAutomatonFrame.getFrame().setSelected(true);
		} catch (java.beans.PropertyVetoException e) {}
	}	
}

}