 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.print.*;

/*
 * Image control used to draw a sentence graph
 * 
 */

class FstGraphicalZone extends JComponent implements KeyListener,MouseListener,MouseMotionListener,Printable {

	public int Width;
	public int Height;
	public FstTextField texte;
	public Vector selectedBoxes;
	public Vector graphBoxes;
	public boolean is_initialised=false;
	public boolean isGrid=false;
	public int nPixels=30;
	public Preferences pref=new Preferences();
	public TextAutomatonFrame parent;
	public double scaleFactor=1.0;
	public boolean antialiasingValue=false;
	
	
	public FstGraphicalZone(int w,int h,FstTextField t,TextAutomatonFrame p) {
	    	super();
	    	Width=w;
	    	Height=h;
	    	texte=t;
	    	parent=p;
	    	selectedBoxes=new Vector();
	    	graphBoxes=new Vector();
	    	setBackground(Color.white);
	    	setLayout(new BorderLayout());
        	addMouseListener(this);
        	addKeyListener(this);
        	pref=Preferences.pref.getClone();
        	antialiasingValue=pref.antialiasing;
	}
	
	
	public TextAutomatonFrame parent() {
		return TextAutomatonFrame.frame;
	}


	public void init() {
	    	for (int i=0;i<graphBoxes.size();i++) {
	    		FstGraphBox g=(FstGraphBox)graphBoxes.get(i);
	    		g.context=this.getGraphics();
	    		g.parent=this;
	    		g.update();
	    	}
	}



/*
 * Methods for adding and creating boxes
 *
 */
	public void addBox(FstGraphBox g) {
		graphBoxes.add(g);
	}


	public FstGraphBox createBox(int x,int y) {
		FstGraphBox g=new FstGraphBox(x,y,2,this.getGraphics(),this);
		g.setText("<E>");
		addBox(g);
		return g;		
	}


	public void selectAllBoxes() {
		unSelectAllBoxes();
		int L=L=graphBoxes.size();
		for (int i=0;i<L;i++) {
			FstGraphBox g=(FstGraphBox)graphBoxes.get(i);
			g.selected=true;
			selectedBoxes.add(g);
			texte.initText(g.contenu);
		}
		texte.setEditable(false);
		repaint();
	}


	public void copySelection(MultipleSelection m) {
		Vector v=m.elem;
		FstGraphBox g;
		GraphBoxInfo tmp;
			
		unSelectAllBoxes();	
		for (int i=0;i<v.size();i++) {
			tmp=(GraphBoxInfo)v.get(i);
			g=createBox(tmp.X+20*m.n,tmp.Y+20*m.n);
			g.setText(tmp.contenu);
			g.selected=true;
			selectedBoxes.add(g);
		}
		for (int i=0;i<selectedBoxes.size();i++) {
			g=(FstGraphBox)selectedBoxes.get(i);
			tmp=tmp=(GraphBoxInfo)v.get(i);
			Vector vec=tmp.tmp;
			int taille=vec.size();
			for (int j=0;j<taille;j++) {
				Integer n=(Integer)vec.get(j);
				g.addTransitionTo((FstGraphBox)selectedBoxes.get(n.intValue()));
			}
		}
		parent().setModified(true);
		texte.initText("");
		repaint();
	}




/*
 * This method remove the selected boxes, updating all the transitions
 *
 */
	public void removeTransitionTo(FstGraphBox dest) {
		int i,L,pos;
		FstGraphBox g;
		if (graphBoxes.isEmpty()) return;
		L=graphBoxes.size();
		for (i=0;i<L;i++) {
			g=(FstGraphBox)graphBoxes.get(i);
			pos=g.transitions.indexOf(dest);
			if (pos!=-1) g.transitions.remove(pos);
		}	
	}


	public void removeTransitionsToSelected() {
		int i,L,pos;
		FstGraphBox g;
		if (selectedBoxes.isEmpty()) return;
		L=selectedBoxes.size();
		for (i=0;i<L;i++) {
			g=(FstGraphBox)selectedBoxes.get(i);
			if (g.TYPE==2) removeTransitionTo(g);
		}	
	}
 
 
	public void removeSelected() {
		int i,L,pos;
		FstGraphBox g;
		if (selectedBoxes.isEmpty()) return;
		L=selectedBoxes.size();
		removeTransitionsToSelected();
		for (i=0;i<L;i++) {
			g=(FstGraphBox)selectedBoxes.get(i);
			if (g.TYPE==2) graphBoxes.remove(g);
		}
		parent().setModified(true);
		unSelectAllBoxes();
	}




/*
 * This method set s as text for all the selected boxes
 *
 */
	public void setTextForSelected(String s) {
		int i,L;
		FstGraphBox g;
		if (selectedBoxes.isEmpty()) return;
		if (s.equals("")) {
			// if we want to destroy the boxes
			removeSelected();
			return;
		}
		L=selectedBoxes.size();
		for (i=0;i<L;i++) {
			g=(FstGraphBox)selectedBoxes.get(i);
			g.setText(s);
		}	
	}



/*
 * Boxes selection methods
 *
 */
	public int getSelectedBox(int x,int y) {
		int i,L;
		FstGraphBox g;
		L=graphBoxes.size();
		for (i=0;i<L;i++) {
			g=(FstGraphBox)graphBoxes.get(i);
			if (x>=g.X && x<=g.X+g.Width && y>=g.Y1 && y<=g.Y1+g.Height)
				return i;
		}
		return -1;
	}


	public void unSelectAllBoxes() {
		FstGraphBox g;
		while (!selectedBoxes.isEmpty()) {
			g=(FstGraphBox)selectedBoxes.get(0);
			g.selected=false;
			selectedBoxes.remove(0);
		}
		texte.setEditable(false);
	}


	public void selectByRectangle(int x,int y,int w,int h) {
		int i,L;
		FstGraphBox g;
		L=graphBoxes.size();
		for (i=0;i<L;i++) {
			g=(FstGraphBox)graphBoxes.get(i);
			if (g.isSelectedByRectangle(x,y,w,h)) {
				g.selected=true;
				selectedBoxes.add(g);
				texte.initText(g.contenu);
			}
		}
	}


	public void translateAll(int dx,int dy) {
		int i,L;
		FstGraphBox g;
		if (selectedBoxes.isEmpty()) return;
		L=selectedBoxes.size();
		for (i=0;i<L;i++) {
			g=(FstGraphBox)selectedBoxes.get(i);
			g.translate(dx,dy);
		}	
	}



/* begin of events handling */


	public int X_start_drag,Y_start_drag;
	public int X_end_drag,Y_end_drag;
	public int X_drag,Y_drag,dragWidth,dragHeight;
	public boolean selecting=false;
	public boolean dragging=false;
	public boolean singleDragging=false;
	public FstGraphBox singleDraggedBox;


    public void mouseClicked(MouseEvent e) {
    	int boxSelected;
    	FstGraphBox b;
    	int x_tmp,y_tmp;
    	
    	if (e.isShiftDown()) {
    		// Shift+click
    		// reverse transitions
   			boxSelected=getSelectedBox((int)(e.getX()/scaleFactor),(int)(e.getY()/scaleFactor));
    		if (boxSelected!=-1) {
    			// if we click on a box
    			b=(FstGraphBox)graphBoxes.get(boxSelected);
    			if (!selectedBoxes.isEmpty()) {
    				// if there are selected boxes, we rely them to the current
    				addReverseTransitions(b);
    				unSelectAllBoxes();
    				parent().setModified(true);
    			}
    		}
    		else {
    			// simple click not on a box
    			texte.initText("");
    			unSelectAllBoxes();
    		}
    	}
    	else if (e.isControlDown()) {
    		// Control+click
    		// creation of a new box
    		b=createBox((int)(e.getX()/scaleFactor),(int)(e.getY()/scaleFactor));
    		parent().setModified(true);
    		// if some boxes are selected, we rely them to the new one
    		if (!selectedBoxes.isEmpty()) {
    			addTransitions(b);
    		}
    		// then, the only selected box is the new one
   			unSelectAllBoxes();
			b.selected=true;
			selectedBoxes.add(b);
			texte.initText("<E>");
    	}
    	else {
   			boxSelected=getSelectedBox((int)(e.getX()/scaleFactor),(int)(e.getY()/scaleFactor));
    		if (boxSelected!=-1) {
    			// if we click on a box
    			b=(FstGraphBox)graphBoxes.get(boxSelected);
    			if (!selectedBoxes.isEmpty()) {
    				// if there are selected boxes, we rely them to the current
    				addTransitions(b);
    				unSelectAllBoxes();
    				parent().setModified(true);
    			}
    			else {
    				// if not, we just select this one
    				b.selected=true;
    				selectedBoxes.add(b);
    				texte.initText(b.contenu);
    			}
    		}
    		else {
    			// simple click not on a box
    			unSelectAllBoxes();
    			texte.initText("");
    			texte.setEditable(false);
    		}
    	}
    	repaint();
		e.consume();
    	return;
    }
    

    public void mousePressed(MouseEvent e) {
    	int selectedBox;
	    addMouseMotionListener(this);
	    if (e.isShiftDown() || e.isAltDown() || e.isControlDown()) return;
	    texte.validateTextField();
        X_start_drag=(int)(e.getX()/scaleFactor);
        Y_start_drag=(int)(e.getY()/scaleFactor);
        X_end_drag=X_start_drag;
        Y_end_drag=Y_start_drag;
        X_drag=X_start_drag;
        Y_drag=Y_start_drag;
        dragWidth=0;
        dragHeight=0;
        selectedBox=getSelectedBox(X_start_drag,Y_start_drag);
        singleDragging=false;
        dragging=false;
        selecting=false;
        if (selectedBox!=-1) {
        	// if we start dragging a box
        	singleDraggedBox=(FstGraphBox)graphBoxes.get(selectedBox);
        	texte.initText(singleDraggedBox.contenu);
        	if (!singleDraggedBox.selected) {
        		dragging=true;
        		singleDragging=true;
        		singleDraggedBox.singleDragging=true;
        	}
        }
        if (!selectedBoxes.isEmpty()) {
        	dragging=true;
        }
        if ((selectedBox==-1) && selectedBoxes.isEmpty()) {
        	// being drawing a selection rectangle
        	dragging=false;
        	selecting=true;
        	texte.initText("");
        }
		repaint();
		e.consume();
    }


    public void mouseReleased(MouseEvent e) {
        removeMouseMotionListener(this);
        if (e.isShiftDown() || e.isAltDown() || e.isControlDown()) return;
	    dragging=false;
	    texte.initText("");
	    texte.setEditable(false);
	    if (singleDragging) {
        	singleDragging=false;
        	singleDraggedBox.singleDragging=false;
        }
        else 
        if (selecting==true) {
        	selectByRectangle(X_drag,Y_drag,dragWidth,dragHeight);
        	texte.setEditable(true);
        	selecting=false;
        }	
		repaint();
		e.consume();
    }


    public void mouseEntered(MouseEvent e) {
    }


    public void mouseExited(MouseEvent e) {
    }


    public void mouseDragged(MouseEvent e) {
    	int Xtmp=X_end_drag;
    	int Ytmp=Y_end_drag;
    	X_end_drag=(int)(e.getX()/scaleFactor);
		Y_end_drag=(int)(e.getY()/scaleFactor);
		int dx=X_end_drag-Xtmp;
		int dy=Y_end_drag-Ytmp;
		
		if (singleDragging) {
			// translates the single dragged box
			singleDraggedBox.translate(dx,dy);
			parent().setModified(true);
       		}
		if (dragging) {
			// translates all the selected boxes
			parent().setModified(true);
			translateAll(dx,dy);		
			// if we were dragging, we have nothing else to do
			repaint();
			e.consume();
			return;	
		}
		
		
		if (X_start_drag<X_end_drag) {
			X_drag=X_start_drag;
			dragWidth=X_end_drag-X_start_drag;
		}
		else {
			X_drag=X_end_drag;
			dragWidth=X_start_drag-X_end_drag;
		}
		if (Y_start_drag<Y_end_drag) {
			Y_drag=Y_start_drag;
			dragHeight=Y_end_drag-Y_start_drag;
		}
		else {
			Y_drag=Y_end_drag;
			dragHeight=Y_start_drag-Y_end_drag;
		}
				
		repaint();
		e.consume();
    }


    public void mouseMoved(MouseEvent e) {
    }


	public void keyTyped(KeyEvent e) {
	}

	public void keyPressed(KeyEvent e) {
	}

	public void keyReleased(KeyEvent e) {
	}






/* end of events handling */





/*
 * Painting the boxes and transitions
 *
 */
	public void addTransitions(FstGraphBox dest) {
		int i,L;
		FstGraphBox g;
		if (selectedBoxes.isEmpty()) return;
		L=selectedBoxes.size();
		for (i=0;i<L;i++) {
			g=(FstGraphBox)selectedBoxes.get(i);
			g.addTransitionTo(dest);
		}	
	}


	public void addReverseTransitions(FstGraphBox from) {
		int i,L;
		FstGraphBox g;
		if (selectedBoxes.isEmpty()) return;
		L=selectedBoxes.size();
		for (i=0;i<L;i++) {
			g=(FstGraphBox)selectedBoxes.get(i);
			from.addTransitionTo(g);
		}	
	}


	public void drawAllTransitions(Graphics gr) {
		int i,L;
		FstGraphBox g;
		if (graphBoxes.isEmpty()) return;
		L=graphBoxes.size();
		for (i=0;i<L;i++) {
			g=(FstGraphBox)graphBoxes.get(i);
			g.drawTransitions(gr);
		}	
	}

	
	public void drawAllBoxes(Graphics gr) {
		int i,L;
		FstGraphBox g;
		if (graphBoxes.isEmpty()) return;
		L=graphBoxes.size();
		for (i=0;i<L;i++) {
			g=(FstGraphBox)graphBoxes.get(i);
			g.draw(gr);
		}	
	}



	public void paintComponent(Graphics f_old) {
		Graphics2D f=(Graphics2D)f_old;
		if (antialiasingValue) {
			f.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
		}
		else {
			f.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_OFF);
		}
		f.setColor(new Color(205,205,205));
		f.fillRect(0,0,getWidth(),getHeight());
		f.setColor(pref.backgroundColor);
		f.fillRect(0,0,Width,Height);
		f.setColor(pref.foregroundColor);
		f.drawRect(10,10,Width-20,Height-20);
		f.drawRect(9,9,Width-18,Height-18);
		f.setColor(pref.foregroundColor);
		if (graphBoxes.size()==0 || graphBoxes.isEmpty()) return;
		if (!is_initialised) {
			this.init();
			is_initialised=true;
		}
		f.setColor(new Color(205,205,205));
		f.fillRect(0,0,getWidth(),getHeight());
		f.setColor(pref.backgroundColor);
		f.fillRect(0,0,Width,Height);
		f.setColor(pref.foregroundColor);
		f.drawRect(10,10,Width-20,Height-20);
		f.drawRect(9,9,Width-18,Height-18);
		f.setColor(pref.foregroundColor);
		drawAllTransitions(f);
		drawAllBoxes(f);
		if (selecting) {
			// here we draw the selection rectangle
			f.setColor(pref.foregroundColor);
			f.drawRect(X_drag,Y_drag,dragWidth,dragHeight);
		}
	}



	public void updateAllBoxes() {
		FstGraphBox g;
		int i;
		for (i=0;i<graphBoxes.size();i++) {
			g=(FstGraphBox)graphBoxes.get(i);
			g.update();
		}
		repaint();
	}


/*
 * This is the print method
 *
 */
	public int print(Graphics g,PageFormat p,int pageIndex) {
		if (pageIndex!=0) return Printable.NO_SUCH_PAGE;
		Graphics2D f=(Graphics2D)g;
		double DPI=96.0;//(double)Toolkit.getDefaultToolkit().getScreenResolution();
		double WidthInInches=p.getImageableWidth()/72;
		double realWidthInInches=(Width/DPI);
		double HeightInInches=p.getImageableHeight()/72;
		double realHeightInInches=(Height/DPI);
		
		double scale_x=WidthInInches/realWidthInInches;
		double scale_y=HeightInInches/realHeightInInches;
		
		f.translate(p.getImageableX(),p.getImageableY());
		if (scale_x<scale_y)
			f.scale(0.99*0.72*scale_x,0.99*0.72*scale_x);
		else f.scale(0.99*0.72*scale_y,0.99*0.72*scale_y);
		f.setColor(pref.backgroundColor);
		f.fillRect(0,0,Width,Height);
		f.setColor(pref.foregroundColor);
		f.drawRect(10,10,Width-20,Height-20);
		f.drawRect(9,9,Width-18,Height-18);
		f.setColor(pref.foregroundColor);
		drawAllTransitions(f);
		drawAllBoxes(f);
		if (selecting) {
			// here we draw the selection rectangle
			f.drawRect(X_drag,Y_drag,dragWidth,dragHeight);
		}
		return Printable.PAGE_EXISTS;
	}







} /* end of FstGraphicalZone */
