 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.io.*;
import java.awt.*;
import java.awt.Toolkit;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.border.*;
import javax.swing.text.TextAction.*;

/*
 * This class describes the text field used to get the box text in a sentence graph
 *
 */
public class FstTextField extends JTextField implements KeyListener {  

	public TextAutomatonFrame parent;
	public boolean modified=false;
	static public String leftContext;
	static public String rightContext;
	public SpecialPaste specialPaste;
	public SpecialCopy specialCopy;
	public SelectAll selectAll;
	public Cut cut;
	public Clipboard clip;
	
	public FstTextField(int n,TextAutomatonFrame p) {
		super(n);
		setEditable(false);
		modified=false;
		parent=p;
		leftContext="";
		rightContext="";
		clip=Toolkit.getDefaultToolkit().getSystemClipboard();
		setDisabledTextColor(Color.white);
		setBackground(Color.white);
		Keymap k=getKeymap();
		k=addKeymap("fsttextfield-keymap",k);
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('v',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('V',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('c',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('C',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('x',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('X',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('a',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('A',Event.CTRL_MASK ));
		
				
		selectAll=new SelectAll("select all");
		cut=new Cut("cut");
		specialPaste=new SpecialPaste("special-paste");
		specialCopy=new SpecialCopy("special-copy");
				
		k.addActionForKeyStroke(KeyStroke.getKeyStroke('v',Event.CTRL_MASK ),specialPaste);
		k.addActionForKeyStroke(KeyStroke.getKeyStroke('V',Event.CTRL_MASK ),specialPaste);
		k.addActionForKeyStroke(KeyStroke.getKeyStroke('c',Event.CTRL_MASK ),specialCopy);
		k.addActionForKeyStroke(KeyStroke.getKeyStroke('C',Event.CTRL_MASK ),specialCopy);
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('m',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('M',Event.CTRL_MASK ));
		k.addActionForKeyStroke(KeyStroke.getKeyStroke('a',Event.CTRL_MASK ),selectAll);
		k.addActionForKeyStroke(KeyStroke.getKeyStroke('A',Event.CTRL_MASK ),selectAll);
		k.addActionForKeyStroke(KeyStroke.getKeyStroke('x',Event.CTRL_MASK ),cut);
		k.addActionForKeyStroke(KeyStroke.getKeyStroke('X',Event.CTRL_MASK ),cut);
		this.setKeymap(k);
				
		addKeyListener(this);
	}


	public TextAutomatonFrame parent() {
		return TextAutomatonFrame.frame;
	}


	class SpecialCopy extends TextAction implements ClipboardOwner {
		public SpecialCopy(String s) {
			super(s);
		}

		public void actionPerformed(ActionEvent e) {
			if (parent().graphicalZone.selectedBoxes.isEmpty() ||
			    parent().graphicalZone.selectedBoxes.size()<2) {
				// is there is no or one box selected, we copy normally
				copy();
				UnitexFrame.clip.setContents(null,this);
				return;
			}
			UnitexFrame.clip.setContents(new MultipleBoxesSelection(new MultipleSelection(parent().graphicalZone.selectedBoxes,true)),this);
		}
		
		public void lostOwnership(Clipboard c,Transferable d) {
		}
	}




	class SelectAll extends TextAction implements ClipboardOwner {
		public SelectAll(String s) {
			super(s);
		}

		public void actionPerformed(ActionEvent e) {
			if (!parent().graphicalZone.selectedBoxes.isEmpty() &&
			    parent().graphicalZone.selectedBoxes.size()==1)
				selectAll();
			else parent().graphicalZone.selectAllBoxes();
		}
		
		public void lostOwnership(Clipboard c,Transferable d) {
		}
	}


	class Cut extends TextAction implements ClipboardOwner {
		public Cut(String s) {
			super(s);
		}

		public void actionPerformed(ActionEvent e) {
			if (!parent().graphicalZone.selectedBoxes.isEmpty() &&
			    parent().graphicalZone.selectedBoxes.size()==1) {
				cut();
			}
			else {
				UnitexFrame.clip.setContents(new MultipleBoxesSelection(new MultipleSelection(parent().graphicalZone.selectedBoxes,true)),this);
				parent().graphicalZone.removeSelected();
				setText("");
			}
		}


		public void lostOwnership(Clipboard c,Transferable d) {
		}
	}


	class SpecialPaste extends TextAction {
		public SpecialPaste(String s) {
			super(s);
		}

		public void actionPerformed(ActionEvent e) {
			Transferable data;
			MultipleSelection res=null;
			data=UnitexFrame.clip.getContents(this);
			try {
				if (data!=null) res=(MultipleSelection)data.getTransferData(new DataFlavor("unitex/boxes","Unitex dataflavor"));
			} catch (UnsupportedFlavorException e2) {e2.printStackTrace();}
			  catch (IOException e2) {e2.printStackTrace();}
			if (res==null || FstTextField.this.modified==true) {
				// if there is no boxes to copy we do a simple paste
				paste();
				return;
			}
			res.n++;
			parent().graphicalZone.copySelection(res);
		}
	}




	public void initText(String s) {
		modified=false;
		setEditable(true);
   		setText(s);
   		requestFocus();
		getCaret().setVisible(true);
		selectAll();
	}
	
	
	public Document createDefaultModel() {
		return new ListDocument();
	}

 
	public boolean validateTextField() {
		if (!hasChangedTextField()) return false;
		if (isGoodText(getText())) {
			parent().graphicalZone.setTextForSelected(getText());
    			parent().graphicalZone.unSelectAllBoxes();
    			setText("");
	    		parent().graphicalZone.repaint();
	    		setEditable(false);
	    		parent().setModified(true);
			return true;
    		}
    	return false;
	}


	public void keyTyped(KeyEvent e) {
	}


	public void keyPressed(KeyEvent e) {
		if (e.isControlDown() || e.isAltDown()) {
			// if the control key or alt key is pressed, we do nothing: the event we be caught by the ActionListeners
			return;
		}
		if (e.getKeyCode()==10) validateTextField();
		modified=true;
	}


	public void keyReleased(KeyEvent e) {
	}


	public boolean hasChangedTextField() {
		return modified;
	}

	
	public boolean isGoodText(String s) {
		if (s.equals("")) return true;
		char ligne[]=new char[10000];
		String tmp="";
		int i,j,L;
		
		ligne=s.toCharArray();
		L=s.length();
		if (ligne[0]!='{') return true;


		i=1;
		while (i<L && ligne[i]!=',') {
			if (ligne[i]=='\\') {
				if (i<L) {
					i++;
				} else {
					JOptionPane.showMessageDialog(null,"Unexpected '\\' at end of line","Error",JOptionPane.ERROR_MESSAGE); 
					return false;
				}
				
			}
			i++;
		}
		if (i==L) {
			JOptionPane.showMessageDialog(null,"No ',' delimiting inflected part from canonical part","Error",JOptionPane.ERROR_MESSAGE); 
			return false;
		}
		i++;
		while (i<L && ligne[i]!='.') {
			if (ligne[i]=='\\') {
				if (i<L) {
					i++;
				} else {
					JOptionPane.showMessageDialog(null,"Unexpected '\\' at end of line","Error",JOptionPane.ERROR_MESSAGE); 
					return false;
				}
				
			}
			i++;
		}
		if (i==L) {
			JOptionPane.showMessageDialog(null,"No '.' delimiting canonical part from grammatical informations","Error",JOptionPane.ERROR_MESSAGE); 
			return false;
		}
		i++;
		while (i<L && ligne[i]!='}') {
			if (ligne[i]=='\\') {
				if (i<L) {
					i++;
				} else {
					JOptionPane.showMessageDialog(null,"Unexpected '\\' at end of line","Error",JOptionPane.ERROR_MESSAGE); 
					return false;
				}
				
			}
			i++;
		}
		if (i==L) {
			JOptionPane.showMessageDialog(null,"No closing '}'","Error",JOptionPane.ERROR_MESSAGE); 
			return false;
		}
   		return true;
	}

}
