 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.border.*;

/*
 * This class describes the graph presentation menu (options, colors & fonts)
 *
 */
public class GraphPresentationMenu extends JDialog {

	JPanel panel=new JPanel();
	JPanel upPanel=new JPanel();
	JPanel downPanel=new JPanel();
	JPanel displayPanel=new JPanel();
	JPanel display1=new JPanel();
	JPanel display2=new JPanel();
	JPanel display3=new JPanel();
	JPanel display4=new JPanel();
	JPanel display5=new JPanel();
	JPanel colorPanel=new JPanel();
	JPanel fontPanel=new JPanel();
	JPanel buttonPanel=new JPanel();
	JCheckBox dateCheckBox=new JCheckBox();
	JCheckBox filenameCheckBox=new JCheckBox();
	JCheckBox pathnameCheckBox=new JCheckBox();
	JCheckBox frameCheckBox=new JCheckBox();
	JCheckBox rightToLeftCheckBox=new JCheckBox();
	JButton color1=new JButton("     ");
	JButton color2=new JButton("     ");
	JButton color3=new JButton("     ");
	JButton color4=new JButton("     ");
	JButton color5=new JButton("     ");
	JButton background=new JButton("Set...");
	JButton foreground=new JButton("Set...");
	JButton subgraph=new JButton("Set...");
	JButton selected=new JButton("Set...");
	JButton comment=new JButton("Set...");
	JButton input=new JButton("Input");
	JButton output=new JButton("Output");
	JButton DEFAULT=new JButton("Default");
	JButton OK=new JButton("OK");
	JButton CANCEL=new JButton("Cancel");
	JLabel label1=new JLabel("  Background:  ",JLabel.LEFT);
	JLabel label2=new JLabel("  Foreground:  ",JLabel.LEFT);
	JLabel label3=new JLabel("  Auxiliary Nodes:  ",JLabel.LEFT);
	JLabel label4=new JLabel("  Selected Nodes:  ",JLabel.LEFT);
	JLabel label5=new JLabel("  Comment Nodes:  ",JLabel.LEFT);
	JLabel inputLabel=new JLabel("",JLabel.LEFT);
	JLabel outputLabel=new JLabel("",JLabel.LEFT);

	public static GraphPresentationMenu pref;

	
	public GraphPresentationMenu() {
		super(UnitexFrame.mainFrame,"Presentation",true);
		Preferences.temp=UnitexFrame.currentFrame.graphicalZone.pref.getClone();
		constructPanel();
		setContentPane(panel);
		pack();
		setResizable(false);
		pref=this;
		showPresentation();	
	}


	public void refresh() {
		// here we refresh the components from the Preferences.temp preferences
		color1.setBackground(Preferences.temp.backgroundColor);
		color2.setBackground(Preferences.temp.foregroundColor);
		color3.setBackground(Preferences.temp.subgraphColor);
		color4.setBackground(Preferences.temp.selectedColor);
		color5.setBackground(Preferences.temp.commentColor);
		inputLabel.setText("  "+Preferences.temp.input.getFontName()+"  "+Preferences.temp.inputSize+"  ");
		outputLabel.setText("  "+Preferences.temp.output.getFontName()+"  "+Preferences.temp.outputSize+"  ");
		setResizable(true);
		pack();
		setResizable(false);
		repaint();
	}


	public void showPresentation() {
		setLocationRelativeTo(UnitexFrame.mainFrame);
		this.setVisible(true);
	}
	
	
	void constructPanel() {
		panel.setBorder(new EmptyBorder(5,5,5,5));
		GridBagLayout g=new GridBagLayout();
		panel.setLayout(g);
		GridBagConstraints c=new GridBagConstraints();
		constructDisplayPanel();
		constructColorPanel();
		constructFontPanel();
		constructButtonPanel();
		constructUpPanel();
		constructDownPanel();
		c.gridwidth=GridBagConstraints.REMAINDER;
		c.fill=GridBagConstraints.BOTH;
		c.anchor=GridBagConstraints.NORTH;
		g.setConstraints(upPanel,c);
		panel.add(upPanel);
		c.fill=GridBagConstraints.BOTH;
		g.setConstraints(downPanel,c);
		panel.add(downPanel);
	}


	void constructUpPanel() {
		upPanel.setBorder(new EmptyBorder(0,0,0,0));
		upPanel.setLayout(new BorderLayout());
		upPanel.add(displayPanel,BorderLayout.WEST);
		upPanel.add(colorPanel,BorderLayout.CENTER);
	}


	void constructDownPanel() {
		downPanel.setBorder(new EmptyBorder(0,0,0,0));
		downPanel.setLayout(new BorderLayout());
		downPanel.add(fontPanel,BorderLayout.CENTER);
		downPanel.add(buttonPanel,BorderLayout.EAST);
	}


	void constructDisplayPanel() {
		displayPanel.setBorder(new TitledBorder("Display"));
		displayPanel.setLayout(new GridLayout(5,1));
		dateCheckBox.setSelected(Preferences.temp.date);
		filenameCheckBox.setSelected(Preferences.temp.filename);
		pathnameCheckBox.setSelected(Preferences.temp.pathname);
		frameCheckBox.setSelected(Preferences.temp.frame);
		rightToLeftCheckBox.setSelected(Preferences.temp.rightToLeft);
		display1.setLayout(new BorderLayout());
		display2.setLayout(new BorderLayout());
		display3.setLayout(new BorderLayout());
		display4.setLayout(new BorderLayout());
		display5.setLayout(new BorderLayout());
		display1.add(dateCheckBox,BorderLayout.WEST);
		display1.add(new JLabel("Date  "),BorderLayout.CENTER);
		display2.add(filenameCheckBox,BorderLayout.WEST);
		display2.add(new JLabel("File Name  "),BorderLayout.CENTER);
		display3.add(pathnameCheckBox,BorderLayout.WEST);
		display3.add(new JLabel("Pathname  "),BorderLayout.CENTER);
		display4.add(frameCheckBox,BorderLayout.WEST);
		display4.add(new JLabel("Frame  "),BorderLayout.CENTER);
		display5.add(rightToLeftCheckBox,BorderLayout.WEST);
		display5.add(new JLabel("Right to Left  "),BorderLayout.CENTER);
		displayPanel.add(display1);
		displayPanel.add(display2);
		displayPanel.add(display3);
		displayPanel.add(display4);
		displayPanel.add(display5);
	}


	void build(GridBagConstraints c,int gx,int gy,int gw,int gh,int wx,int wy) {
		c.gridx=gx;
		c.gridy=gy;
		c.gridwidth=gw;
		c.gridheight=gh;
		c.weightx=wx;
		c.weighty=wy;
	}
	

	public void constructColorPanel() {
		colorPanel.setBorder(new TitledBorder("Colors"));
		GridBagLayout g=new GridBagLayout();
		colorPanel.setLayout(g);
		background.addMouseListener(new Color1Listener());
		foreground.addMouseListener(new Color2Listener());
		subgraph.addMouseListener(new Color3Listener());
		selected.addMouseListener(new Color4Listener());
		comment.addMouseListener(new Color5Listener());
		color1.setBackground(Preferences.temp.backgroundColor);
		color2.setBackground(Preferences.temp.foregroundColor);
		color3.setBackground(Preferences.temp.subgraphColor);
		color4.setBackground(Preferences.temp.selectedColor);
		color5.setBackground(Preferences.temp.commentColor);
		GridBagConstraints c=new GridBagConstraints();
		build(c,0,0,1,1,60,15);
		c.fill=GridBagConstraints.NONE;
		c.anchor=GridBagConstraints.EAST;
		g.setConstraints(label1,c);
		colorPanel.add(label1);
		build(c,1,0,1,1,20,0);
		c.fill=GridBagConstraints.HORIZONTAL;
		g.setConstraints(color1,c);
		colorPanel.add(color1);
		build(c,2,0,1,1,20,0);
		c.fill=GridBagConstraints.HORIZONTAL;
		g.setConstraints(background,c);
		colorPanel.add(background);
		build(c,0,1,1,1,60,15);
		c.fill=GridBagConstraints.NONE;
		c.anchor=GridBagConstraints.EAST;
		g.setConstraints(label2,c);
		colorPanel.add(label2);
		build(c,1,1,1,1,20,0);
		c.fill=GridBagConstraints.HORIZONTAL;
		g.setConstraints(color2,c);
		colorPanel.add(color2);
		build(c,2,1,1,1,20,0);
		c.fill=GridBagConstraints.HORIZONTAL;
		g.setConstraints(foreground,c);
		colorPanel.add(foreground);
		build(c,0,2,1,1,60,15);
		c.fill=GridBagConstraints.NONE;
		c.anchor=GridBagConstraints.EAST;
		g.setConstraints(label3,c);
		colorPanel.add(label3);
		build(c,1,2,1,1,20,0);
		c.fill=GridBagConstraints.HORIZONTAL;
		g.setConstraints(color3,c);
		colorPanel.add(color3);
		build(c,2,2,1,1,20,0);
		c.fill=GridBagConstraints.HORIZONTAL;
		g.setConstraints(subgraph,c);
		colorPanel.add(subgraph);
		build(c,0,3,1,1,60,15);
		c.fill=GridBagConstraints.NONE;
		c.anchor=GridBagConstraints.EAST;
		g.setConstraints(label4,c);
		colorPanel.add(label4);
		build(c,1,3,1,1,20,0);
		c.fill=GridBagConstraints.HORIZONTAL;
		g.setConstraints(color4,c);
		colorPanel.add(color4);
		build(c,2,3,1,1,20,0);
		c.fill=GridBagConstraints.HORIZONTAL;
		g.setConstraints(selected,c);
		colorPanel.add(selected);
		build(c,0,4,1,1,60,15);
		c.fill=GridBagConstraints.NONE;
		c.anchor=GridBagConstraints.EAST;
		g.setConstraints(label5,c);
		colorPanel.add(label5);
		build(c,1,4,1,1,20,0);
		c.fill=GridBagConstraints.HORIZONTAL;
		g.setConstraints(color5,c);
		colorPanel.add(color5);
		build(c,2,4,1,1,20,0);
		c.fill=GridBagConstraints.HORIZONTAL;
		g.setConstraints(comment,c);
		colorPanel.add(comment);
	}
	
	
	public void constructFontPanel() {
		fontPanel.setBorder(new TitledBorder("Fonts"));
		GridBagLayout g=new GridBagLayout();
		fontPanel.setLayout(g);
		GridBagConstraints c=new GridBagConstraints();
		input.addMouseListener(new inputListener());
		inputLabel.setText("  "+Preferences.temp.input.getFontName()+"  "+Preferences.temp.inputSize+"  ");
		output.addMouseListener(new outputListener());
		outputLabel.setText("  "+Preferences.temp.output.getFontName()+"  "+Preferences.temp.outputSize+"  ");
		build(c,0,0,1,1,60,15);
		c.fill=GridBagConstraints.BOTH;
		c.anchor=GridBagConstraints.WEST;
		g.setConstraints(input,c);
		fontPanel.add(input);
		build(c,1,0,1,1,20,0);
		c.fill=GridBagConstraints.VERTICAL;
		c.anchor=GridBagConstraints.WEST;
		g.setConstraints(inputLabel,c);
		fontPanel.add(inputLabel);
		build(c,0,1,1,1,60,15);
		c.fill=GridBagConstraints.BOTH;
		c.anchor=GridBagConstraints.WEST;
		g.setConstraints(output,c);
		fontPanel.add(output);
		build(c,1,1,1,1,20,0);
		c.fill=GridBagConstraints.VERTICAL;
		c.anchor=GridBagConstraints.WEST;
		g.setConstraints(outputLabel,c);
		fontPanel.add(outputLabel);
	}
	
	
	public void constructButtonPanel() {
		buttonPanel.setBorder(new EmptyBorder(5,5,0,0));
		buttonPanel.setLayout(new GridLayout(3,1));
		DEFAULT.addMouseListener(new DEFAULTListener());
		OK.addMouseListener(new OKListener());
		CANCEL.addMouseListener(new CANCELListener());
		buttonPanel.add(DEFAULT);
		buttonPanel.add(OK);
		buttonPanel.add(CANCEL);
	}


class DEFAULTListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		if (UnitexFrame.currentFrame==null) return;
		Preferences.temp=Preferences.pref.getClone();
		GraphPresentationMenu.pref.refresh();
		dateCheckBox.setSelected(Preferences.temp.date);
		filenameCheckBox.setSelected(Preferences.temp.filename);
		pathnameCheckBox.setSelected(Preferences.temp.pathname);
		frameCheckBox.setSelected(Preferences.temp.frame);
		rightToLeftCheckBox.setSelected(Preferences.temp.rightToLeft);
		e.consume();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class OKListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		if (UnitexFrame.currentFrame==null) return;
		Dimension d=UnitexFrame.currentFrame.getSize(null);
		Preferences.temp.date=GraphPresentationMenu.pref.dateCheckBox.isSelected();
		Preferences.temp.filename=GraphPresentationMenu.pref.filenameCheckBox.isSelected();
		Preferences.temp.pathname=GraphPresentationMenu.pref.pathnameCheckBox.isSelected();
		Preferences.temp.frame=GraphPresentationMenu.pref.frameCheckBox.isSelected();
		Preferences.temp.rightToLeft=GraphPresentationMenu.pref.rightToLeftCheckBox.isSelected();
		UnitexFrame.currentFrame.graphicalZone.pref=Preferences.temp.getClone();
		UnitexFrame.currentFrame.texte.setFont(UnitexFrame.currentFrame.graphicalZone.pref.input);
		UnitexFrame.currentFrame.pack();
		UnitexFrame.currentFrame.setSize(d);
		UnitexFrame.currentFrame.graphicalZone.updateAllBoxes();
		GraphPresentationMenu.pref.setVisible(false);
		UnitexFrame.currentFrame.setModified(true);
		e.consume();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class CANCELListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		GraphPresentationMenu.pref.setVisible(false);
		e.consume();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class inputListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		new FontMenu(true);
		GraphPresentationMenu.pref.refresh();
		e.consume();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class outputListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		new FontMenu(false);
		GraphPresentationMenu.pref.refresh();
		e.consume();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class Color1Listener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		Preferences.temp.backgroundColor=JColorChooser.showDialog(UnitexFrame.mainFrame,
						"Background Color",
						Preferences.temp.backgroundColor);
        GraphPresentationMenu.pref.refresh();
		e.consume();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class Color2Listener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		Preferences.temp.foregroundColor=JColorChooser.showDialog(UnitexFrame.mainFrame,
						"Foreground Color",
						Preferences.temp.foregroundColor);
		GraphPresentationMenu.pref.refresh();
		e.consume();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class Color3Listener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		Preferences.temp.subgraphColor=JColorChooser.showDialog(UnitexFrame.mainFrame,
						"Auxiliary Nodes Color",
						Preferences.temp.subgraphColor);
		GraphPresentationMenu.pref.refresh();
		e.consume();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class Color4Listener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		Preferences.temp.selectedColor=JColorChooser.showDialog(UnitexFrame.mainFrame,
						"Selected Nodes Color",
						Preferences.temp.selectedColor);
		GraphPresentationMenu.pref.refresh();
		e.consume();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class Color5Listener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		Preferences.temp.commentColor=JColorChooser.showDialog(UnitexFrame.mainFrame,
						"Comment Nodes Color",
						Preferences.temp.commentColor);
		GraphPresentationMenu.pref.refresh();
		e.consume();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}



}


