 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.border.*;

/*
 * This class describes the graph size menu
 *
 */
public class GraphSizeMenu extends JDialog {

	JPanel panel=new JPanel();
	JPanel panel1=new JPanel();
	JPanel downPanel=new JPanel();
	JPanel panel2=new JPanel();
	JPanel panel3=new JPanel();
	JPanel buttonPanel=new JPanel();
	SpecialNumericTextField Width=new SpecialNumericTextField(4,String.valueOf(0));
	SpecialNumericTextField Height=new SpecialNumericTextField(4,String.valueOf(0));
	Choice unitList=new Choice();
	JButton A4=new JButton("Set to A4");
	JButton orientation=new JButton("Portrait/Lanscape");
	JButton OK=new JButton("OK");
	JButton CANCEL=new JButton("Cancel");
	
	static int resolutionDPI=Toolkit.getDefaultToolkit().getScreenResolution();
	
	public static GraphSizeMenu pref;
	float X;
	float Y;
	static final int PIXELS=1;
	static final int INCHES=2;
	static final int CM=3;
	static int unit=PIXELS;
	
	
	public GraphSizeMenu() {
		super(UnitexFrame.mainFrame,"Graph Size",true);
		pref=this;
		constructPanel();
		setContentPane(panel);
		X=UnitexFrame.currentFrame.graphicalZone.Width;
		Y=UnitexFrame.currentFrame.graphicalZone.Height;
		Width.setText(stringValueOfX());
		Height.setText(stringValueOfY());
		unit=PIXELS;
		pack();
		setResizable(false);
		pref=this;
		showMenu();
	}


	public float getValueOfX() {
		if (unit==PIXELS) return (float)X;
		else if (unit==INCHES) return (float)(X/resolutionDPI);
		else return (float)((X*24/resolutionDPI)/10);
	}


	public float getValueOfY() {
		if (unit==PIXELS) return (float)Y;
		else if (unit==INCHES) return (float)(Y/resolutionDPI);
		else return (float)((Y*24/resolutionDPI)/10);
	}


	public String stringValueOfX() {
		if (unit==PIXELS) return String.valueOf((int)getValueOfX());
		else return String.valueOf((float)((int)(getValueOfX()*100))/100);
	}


	public String stringValueOfY() {
		if (unit==PIXELS) return String.valueOf((int)getValueOfY());
		else return String.valueOf((float)((int)(getValueOfY()*100))/100);
	}
	

	public void showMenu() {
		setLocationRelativeTo(UnitexFrame.mainFrame);
		this.setVisible(true);
	}
	
	
	void constructPanel() {
		panel.setBorder(new EmptyBorder(10,10,10,10));
		panel.setLayout(new BorderLayout());
		constructPanel1();
		constructDownPanel();
		panel.add(panel1,BorderLayout.NORTH);
		panel.add(downPanel,BorderLayout.CENTER);
	}


	void constructPanel1() {
		panel1.setBorder(new EmptyBorder(5,5,5,5));
		panel1.setLayout(new FlowLayout());
		panel1.add(new JLabel("Width : "));
		Width.setEditable(true);
		panel1.add(Width);
		panel1.add(new JLabel(" x Height : "));
		Height.setEditable(true);
		panel1.add(Height);
	}


	void constructDownPanel() {
		downPanel.setLayout(new BorderLayout());
		constructPanel2();
		constructPanel3();
		downPanel.add(panel2,BorderLayout.WEST);
		downPanel.add(panel3,BorderLayout.CENTER);
	}


	void constructPanel2() {
		panel2.setBorder(new EmptyBorder(5,5,5,5));
		panel2.setLayout(new GridLayout(3,1));
		panel2.add(new JLabel("Unit :"));
		unitList.addItem("Pixels");
		unitList.addItem("Inches");
		unitList.addItem("Cm");
		unitList.addItemListener(new UnitListener());
		unitList.select(0);
		panel2.add(unitList);
		A4.addMouseListener(new A4Listener());
		panel2.add(A4);
	}


	void constructPanel3() {
		panel3.setBorder(new EmptyBorder(5,5,5,5));
		panel3.setLayout(new GridLayout(3,1));
		constructButtonPanel();
		panel3.add(new JLabel("Orientation :"));
		orientation.addMouseListener(new OrientationListener());
		panel3.add(orientation);
		panel3.add(buttonPanel);
	}


	void constructButtonPanel() {
		buttonPanel.setLayout(new GridLayout(1,2));
		OK.addMouseListener(new OKListener());
		CANCEL.addMouseListener(new CANCELListener());
		buttonPanel.add(OK);
		buttonPanel.add(CANCEL);
	}
	

	class SpecialNumericTextField extends JTextField {
		
		SpecialNumericTextField(int c,String s) {
			super(c);
			setEditable(true);
			setText(s);
			setEditable(false);
		}
		
		public Document createDefaultModel() {
			return new SpecialNumericTextDocument();
		}
	}


	class SpecialNumericTextDocument extends PlainDocument {
		public void insertString(int offs,String s,AttributeSet a) throws BadLocationException {
			int i;
			if (s==null) return;
			char c[]=s.toCharArray();
			for (i=0;i<c.length;i++) {
				if ((c[i]<'0' || c[i]>'9') && c[i]!='.') return;
				if ((GraphSizeMenu.unit==PIXELS) && (c[i]=='.')) return;
			}
			super.insertString(offs,s,a);
		}
	}
	
	
	
	class A4Listener implements MouseListener {
		public void mousePressed(MouseEvent e) {}
		public void mouseReleased(MouseEvent e) {
			e.consume();
			X=(float)((29.7*resolutionDPI)/2.4);
			Y=(float)((21*resolutionDPI)/2.4);
			Width.setText(stringValueOfX());
			Height.setText(stringValueOfY());
			repaint();
		}
		public void mouseMoved(MouseEvent e) {}
		public void mouseDragged(MouseEvent e) {}
		public void mouseEntered(MouseEvent e) {}
		public void mouseExited(MouseEvent e) {}
		public void mouseClicked(MouseEvent e) {}
	}


	class OrientationListener implements MouseListener {
		public void mousePressed(MouseEvent e) {}
		public void mouseReleased(MouseEvent e) {
			e.consume();
			float tmp;
			tmp=X;
			X=Y;
			Y=tmp;
			Width.setText(stringValueOfX());
			Height.setText(stringValueOfY());
			repaint();
		}
		public void mouseMoved(MouseEvent e) {}
		public void mouseDragged(MouseEvent e) {}
		public void mouseEntered(MouseEvent e) {}
		public void mouseExited(MouseEvent e) {}
		public void mouseClicked(MouseEvent e) {}
	}


	class UnitListener implements ItemListener {
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange()!=ItemEvent.SELECTED) return;
			float x,y;
			try {
				x=(new Float(Width.getText())).floatValue();
				y=(new Float(Height.getText())).floatValue();
			} catch (NumberFormatException z) {
				JOptionPane.showMessageDialog(null,"Invalid value","Error",JOptionPane.ERROR_MESSAGE); 
				return;
			}
			if (unit==PIXELS) {
				X=x;
				Y=y;
			} 
			else if (unit==INCHES) {
				X=(float)((x*resolutionDPI));
				Y=(float)((y*resolutionDPI));
			}
			else {
				X=(float)((x*resolutionDPI)/2.4);
				Y=(float)((y*resolutionDPI)/2.4);
			}
			
			unit=unitList.getSelectedIndex()+1;
			Width.setText(stringValueOfX());
			Height.setText(stringValueOfY());
			repaint();
		}
	}


	class OKListener implements MouseListener {
		public void mousePressed(MouseEvent e) {}
		public void mouseReleased(MouseEvent e) {
			e.consume();
						float x,y;
			try {
				x=(new Float(Width.getText())).floatValue();
				y=(new Float(Height.getText())).floatValue();
			} catch (NumberFormatException z) {
				JOptionPane.showMessageDialog(null,"Invalid value","Error",JOptionPane.ERROR_MESSAGE); 
				return;
			}
			if (unit==PIXELS) {
				X=x;
				Y=y;
			} 
			else if (unit==INCHES) {
				X=(float)((x*resolutionDPI));
				Y=(float)((y*resolutionDPI));
			}
			else {
				X=(float)((x*resolutionDPI)/2.4);
				Y=(float)((y*resolutionDPI)/2.4);
			}
			UnitexFrame.currentFrame.reSize((int)X,(int)Y);
			UnitexFrame.currentFrame.setModified(true);
			GraphSizeMenu.pref.setVisible(false);
		}
		public void mouseMoved(MouseEvent e) {}
		public void mouseDragged(MouseEvent e) {}
		public void mouseEntered(MouseEvent e) {}
		public void mouseExited(MouseEvent e) {}
		public void mouseClicked(MouseEvent e) {}
	}


	class CANCELListener implements MouseListener {
		public void mousePressed(MouseEvent e) {}
		public void mouseReleased(MouseEvent e) {
			e.consume();
			GraphSizeMenu.pref.setVisible(false);
		}
		public void mouseMoved(MouseEvent e) {}
		public void mouseDragged(MouseEvent e) {}
		public void mouseEntered(MouseEvent e) {}
		public void mouseExited(MouseEvent e) {}
		public void mouseClicked(MouseEvent e) {}
	}

}