 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.print.*;

/*
 * Image control used to draw a graph
 *
 * 
 */

class GraphicalZone extends JComponent implements KeyListener,MouseListener,MouseMotionListener,Printable {

	public int Width;
	public int Height;
	public TextField texte;
	public Vector selectedBoxes;
	public Vector graphBoxes;
	public boolean is_initialised=false;
	public boolean isGrid=false;
	public int nPixels=30;
	public Preferences pref=new Preferences();
	public GraphFrame parent;
	public double scaleFactor=1.0;
	public int Xmouse,Ymouse;
	public boolean mouseInGraphicalZone=false;
	
	public int EDITING_MODE=MyCursors.NORMAL;
	
	public GraphicalZone(int w,int h,TextField t,GraphFrame p) {
	    	super();
	    	Width=w;
	    	Height=h;
	    	texte=t;
	    	parent=p;
	    	selectedBoxes=new Vector();
	    	graphBoxes=new Vector();
	    	setBackground(Color.white);
	    	setLayout(new BorderLayout());
        	addMouseListener(this);
        	addMouseMotionListener(this);
        	addKeyListener(this);
        	texte.setEditable(false);
        	pref=Preferences.pref.getClone();
	}
	
	
	public GraphFrame parent() {
		return UnitexFrame.currentFrame;
	}


	public void init() {
		if (!parent().empty) {
			int i=0;
			GraphBox g,g2;
			// creating the final state
			g=new GraphBox(300,200,1,this.getGraphics(),this);
			g.setText("<E>");
			// and the initial state
		   	g2=new GraphBox(70,200,0,this.getGraphics(),this);
		   	g2.n_lignes=0;
		   	g2.setText("<E>");
		    addBox(g2);
		    addBox(g);
		    texte.initText("");
		    texte.setEditable(false);
	    } else {
	    	for (int i=0;i<graphBoxes.size();i++) {
	    		GraphBox g=(GraphBox)graphBoxes.get(i);
	    		g.context=this.getGraphics();
	    		g.parent=this;
	    		g.update();
	    	}
	    }
	}



/*
 * Methods for adding and creating boxes
 *
 */
	public void addBox(GraphBox g) {
		graphBoxes.add(g);
	}


	public GraphBox createBox(int x,int y) {
		GraphBox g=new GraphBox(x,y,2,this.getGraphics(),this);
		g.setText("<E>");
		addBox(g);
		return g;		
	}


	public void copySelection(MultipleSelection m) {
		Vector v=m.elem;
		GraphBox g;
		GraphBoxInfo tmp;
			
		unSelectAllBoxes();	
		for (int i=0;i<v.size();i++) {
			tmp=(GraphBoxInfo)v.get(i);
			g=createBox(tmp.X+20*m.n,tmp.Y+20*m.n);
			g.setText(tmp.contenu);
			g.selected=true;
			selectedBoxes.add(g);
		}
		for (int i=0;i<selectedBoxes.size();i++) {
			g=(GraphBox)selectedBoxes.get(i);
			tmp=tmp=(GraphBoxInfo)v.get(i);
			Vector vec=tmp.tmp;
			int taille=vec.size();
			for (int j=0;j<taille;j++) {
				Integer n=(Integer)vec.get(j);
				g.addTransitionTo((GraphBox)selectedBoxes.get(n.intValue()));
			}
		}
		parent().setModified(true);
		texte.initText("");
		repaint();
	}




/*
 * This method remove the selected boxes, updating all the transitions
 *
 */
	public void removeTransitionTo(GraphBox dest) {
		int i,L,pos;
		GraphBox g;
		if (graphBoxes.isEmpty()) return;
		L=graphBoxes.size();
		for (i=0;i<L;i++) {
			g=(GraphBox)graphBoxes.get(i);
			pos=g.transitions.indexOf(dest);
			if (pos!=-1) g.transitions.remove(pos);
		}	
	}


	public void removeTransitionsToSelected() {
		int i,L,pos;
		GraphBox g;
		if (selectedBoxes.isEmpty()) return;
		L=selectedBoxes.size();
		for (i=0;i<L;i++) {
			g=(GraphBox)selectedBoxes.get(i);
			if (g.TYPE==2) removeTransitionTo(g);
		}	
	}
 
 
	public void removeSelected() {
		int i,L,pos;
		GraphBox g;
		if (selectedBoxes.isEmpty()) return;
		L=selectedBoxes.size();
		removeTransitionsToSelected();
		for (i=0;i<L;i++) {
			g=(GraphBox)selectedBoxes.get(i);
			if (g.TYPE==2) graphBoxes.remove(g);
		}	
		unSelectAllBoxes();
		repaint();
	}




/*
 * This method set s as text for all the selected boxes
 *
 */
	public void setTextForSelected(String s) {
		int i,L;
		GraphBox g;
		if (selectedBoxes.isEmpty()) return;
		if (s.equals("")) {
			// if we want to destroy the boxes
			removeSelected();
			return;
		}
		L=selectedBoxes.size();
		for (i=0;i<L;i++) {
			g=(GraphBox)selectedBoxes.get(i);
			g.setText(s);
		}	
	}



/*
 * Boxes selection methods
 *
 */
	public int getSelectedBox(int x,int y) {
		int i,L;
		GraphBox g;
		L=graphBoxes.size();
		for (i=0;i<L;i++) {
			g=(GraphBox)graphBoxes.get(i);
			if (x>=g.X && x<=g.X+g.Width && y>=g.Y1 && y<=g.Y1+g.Height)
				return i;
		}
		return -1;
	}


	public void unSelectAllBoxes() {
		GraphBox g;
		while (!selectedBoxes.isEmpty()) {
			g=(GraphBox)selectedBoxes.get(0);
			g.selected=false;
			selectedBoxes.remove(0);
		}
		texte.setEditable(false);
	}



	public void selectAllBoxes() {
		unSelectAllBoxes();
		int L=L=graphBoxes.size();
		for (int i=0;i<L;i++) {
			GraphBox g=(GraphBox)graphBoxes.get(i);
			g.selected=true;
			selectedBoxes.add(g);
			texte.initText(g.contenu);
		}
		texte.setEditable(false);
		repaint();
	}



	public void selectByRectangle(int x,int y,int w,int h) {
		int i,L;
		GraphBox g;
		L=graphBoxes.size();
		for (i=0;i<L;i++) {
			g=(GraphBox)graphBoxes.get(i);
			if (g.isSelectedByRectangle(x,y,w,h)) {
				g.selected=true;
				selectedBoxes.add(g);
				texte.initText(g.contenu);
			}
		}
	}


	public void translateAll(int dx,int dy) {
		int i,L;
		GraphBox g;
		if (selectedBoxes.isEmpty()) return;
		L=selectedBoxes.size();
		for (i=0;i<L;i++) {
			g=(GraphBox)selectedBoxes.get(i);
			g.translate(dx,dy);
		}	
	}



/* begin of events handling */


	public int X_start_drag,Y_start_drag;
	public int X_end_drag,Y_end_drag;
	public int X_drag,Y_drag,dragWidth,dragHeight;
	public boolean selecting=false;
	public boolean dragging=false;
	public boolean singleDragging=false;
	public GraphBox singleDraggedBox;


    public void mouseClicked(MouseEvent e) {
    	int boxSelected;
    	GraphBox b;
    	int x_tmp,y_tmp;
    	
    	if (EDITING_MODE==MyCursors.REVERSE_LINK_BOXES || (EDITING_MODE==MyCursors.NORMAL && e.isShiftDown())) {
    		// Shift+click
    		// reverse transitions
   		boxSelected=getSelectedBox((int)(e.getX()/scaleFactor),(int)(e.getY()/scaleFactor));
    		if (boxSelected!=-1) {
    			// if we click on a box
    			b=(GraphBox)graphBoxes.get(boxSelected);
  			if (!selectedBoxes.isEmpty()) {
    				// if there are selected boxes, we rely them to the current
    				addReverseTransitions(b);
    				unSelectAllBoxes();
    				parent().setModified(true);
    			}
    			else {
    				if (EDITING_MODE==MyCursors.REVERSE_LINK_BOXES) {
    					// if we click on a box while there is no box selected in REVERSE_LINK_BOXES mode, 
    					// we select it
    					b.selected=true;
    					selectedBoxes.add(b);
    					texte.initText(b.contenu);
    				}
    			}
    		}
    		else {  
			// simple click not on a box
			texte.initText("");
			unSelectAllBoxes();
		}
    	}
    	else if (EDITING_MODE==MyCursors.CREATE_BOXES || (EDITING_MODE==MyCursors.NORMAL && e.isControlDown())) {
    		// Control+click
    		// creation of a new box
    		b=createBox((int)(e.getX()/scaleFactor),(int)(e.getY()/scaleFactor));
    		parent().setModified(true);
    		// if some boxes are selected, we rely them to the new one
    		if (!selectedBoxes.isEmpty()) {
    			addTransitions(b);
    		}
    		// then, the only selected box is the new one
   			unSelectAllBoxes();
			b.selected=true;
			selectedBoxes.add(b);
			texte.initText("<E>");
    	}
    	else if (EDITING_MODE==MyCursors.OPEN_SUBGRAPH || (EDITING_MODE==MyCursors.NORMAL && e.isAltDown())) {
    		// Alt+click
    		// opening of a sub-graph
    		x_tmp=(int)(e.getX()/scaleFactor);
    		y_tmp=(int)(e.getY()/scaleFactor);
   		boxSelected=getSelectedBox(x_tmp,y_tmp);
    		if (boxSelected!=-1) {
    			// if we click on a box
    			b=(GraphBox)graphBoxes.get(boxSelected);
    			String s=b.getGraphClicked(y_tmp);
    			if (!s.equals("")) {
    				// if a graph call was clicked, we show it
    				if (s.lastIndexOf('.')==-1 || s.lastIndexOf('.')!=s.length()-4) {
    					s=s.concat(".grf");
    				}
    				File fichier=new File(s);
    				String fullName,path,name;
    				if (fichier.isAbsolute()) {
    					fullName=fichier.getAbsolutePath();
    					path=fichier.getParent();
    					name=fichier.getName();
    				}
    				else {
    					s=parent().pathname+File.separatorChar+s;
    					fichier=new File(s);
    					fullName=fichier.getAbsolutePath();
    					path=fichier.getParent();
    					name=fichier.getName();
    				}
    				UnitexFrame.mainFrame.loadGraph(fullName,path,name);
    			}
    		}
    	}
    	else if (EDITING_MODE==MyCursors.KILL_BOXES) {
    		// killing a box
    		if (!selectedBoxes.isEmpty()) {
    			// if boxes are selected, we remove them
    			removeSelected();
    		}
    		else {
    			// else, we check if we clicked on a box
    			x_tmp=(int)(e.getX()/scaleFactor);
    			y_tmp=(int)(e.getY()/scaleFactor);
   			boxSelected=getSelectedBox(x_tmp,y_tmp);
    			if (boxSelected!=-1) {
    				b=(GraphBox)graphBoxes.get(boxSelected);
    				b.selected=true;
				selectedBoxes.add(b);
				removeSelected();
    			}
    		}
    	}
    	else {
   		boxSelected=getSelectedBox((int)(e.getX()/scaleFactor),(int)(e.getY()/scaleFactor));
    		if (boxSelected!=-1) {
    			// if we click on a box
    			b=(GraphBox)graphBoxes.get(boxSelected);
    			if (!selectedBoxes.isEmpty()) {
    				// if there are selected boxes, we rely them to the current one
    				addTransitions(b);
    				unSelectAllBoxes();
    				parent().setModified(true);
    			}
    			else {
    				if (!((EDITING_MODE==MyCursors.LINK_BOXES) && (b.TYPE==1))) {
    					// if not, we just select this one, but only if we are not clicking 
    					// on final state in LINK_BOXES mode
    					b.selected=true;
    					selectedBoxes.add(b);
    					texte.initText(b.contenu);
    				}
    			}
    		}
    		else {
    			// simple click not on a box
    			unSelectAllBoxes();
    			texte.initText("");
    			texte.setEditable(false);
    		}
    	}
    	repaint();
	e.consume();
    	return;
    }
    

    public void mousePressed(MouseEvent e) {
    	int selectedBox;
	    //addMouseMotionListener(this);
	    if ((EDITING_MODE==MyCursors.NORMAL && (e.isShiftDown() || e.isAltDown() || e.isControlDown()))
	        || (EDITING_MODE==MyCursors.OPEN_SUBGRAPH)
	        || (EDITING_MODE==MyCursors.KILL_BOXES)
	        ) {
	        return;
	    }
	texte.validateTextField();
        X_start_drag=(int)(e.getX()/scaleFactor);
        Y_start_drag=(int)(e.getY()/scaleFactor);
        X_end_drag=X_start_drag;
        Y_end_drag=Y_start_drag;
        X_drag=X_start_drag;
        Y_drag=Y_start_drag;
        dragWidth=0;
        dragHeight=0;
        selectedBox=getSelectedBox(X_start_drag,Y_start_drag);
        singleDragging=false;
        dragging=false;
        selecting=false;
        if (selectedBox!=-1) {
        	// if we start dragging a box
        	singleDraggedBox=(GraphBox)graphBoxes.get(selectedBox);
        	texte.initText(singleDraggedBox.contenu);
        	if (!singleDraggedBox.selected) {
        		dragging=true;
        		singleDragging=true;
        		singleDraggedBox.singleDragging=true;
        	}
        }
        if (!selectedBoxes.isEmpty()) {
        	dragging=true;
        }
        if ((selectedBox==-1) && selectedBoxes.isEmpty()) {
        	// being drawing a selection rectangle
        	dragging=false;
        	selecting=true;
        	texte.initText("");
        }
		repaint();
		e.consume();
    }


    public void mouseReleased(MouseEvent e) {
        //removeMouseMotionListener(this);
        if (e.isShiftDown() || e.isAltDown() || e.isControlDown()) return;
	    dragging=false;
	    texte.initText("");
	    texte.setEditable(false);
	    if (singleDragging) {
        	singleDragging=false;
        	singleDraggedBox.singleDragging=false;
        }
        else 
        if (selecting==true) {
        	selectByRectangle(X_drag,Y_drag,dragWidth,dragHeight);
        	texte.setEditable(true);
        	selecting=false;
        }	
		repaint();
		e.consume();
    }


    public void mouseEntered(MouseEvent e) {
    	mouseInGraphicalZone=true;
    	repaint();
    }


    public void mouseExited(MouseEvent e) {
    	mouseInGraphicalZone=false;
    	repaint();
    }


    public void mouseDragged(MouseEvent e) {
    	int Xtmp=X_end_drag;
    	int Ytmp=Y_end_drag;
    	X_end_drag=(int)(e.getX()/scaleFactor);
	Y_end_drag=(int)(e.getY()/scaleFactor);
		int dx=X_end_drag-Xtmp;
		int dy=Y_end_drag-Ytmp;
		
		if (singleDragging) {
			// translates the single dragged box
			singleDraggedBox.translate(dx,dy);
			parent().setModified(true);
       		}
		if (dragging && EDITING_MODE==MyCursors.NORMAL) {
			// translates all the selected boxes
			parent().setModified(true);
			translateAll(dx,dy);		
			// if we were dragging, we have nothing else to do
			repaint();
			e.consume();
			return;	
		}
		
		
		if (X_start_drag<X_end_drag) {
			X_drag=X_start_drag;
			dragWidth=X_end_drag-X_start_drag;
		}
		else {
			X_drag=X_end_drag;
			dragWidth=X_start_drag-X_end_drag;
		}
		if (Y_start_drag<Y_end_drag) {
			Y_drag=Y_start_drag;
			dragHeight=Y_end_drag-Y_start_drag;
		}
		else {
			Y_drag=Y_end_drag;
			dragHeight=Y_start_drag-Y_end_drag;
		}
				
		repaint();
		e.consume();
    }


    public void mouseMoved(MouseEvent e) {
    	Xmouse=(int)(e.getX()/scaleFactor);
    	Ymouse=(int)(e.getY()/scaleFactor);
    	if ((EDITING_MODE==MyCursors.REVERSE_LINK_BOXES || EDITING_MODE==MyCursors.LINK_BOXES) &&
    		!selectedBoxes.isEmpty()) {
    		repaint();
	}
    }


	public void keyTyped(KeyEvent e) {
	}

	public void keyPressed(KeyEvent e) {
	}

	public void keyReleased(KeyEvent e) {
	}






/* end of events handling */





/*
 * Painting the boxes and transitions
 *
 */
	public void addTransitions(GraphBox dest) {
		int i,L;
		GraphBox g;
		if (selectedBoxes.isEmpty()) return;
		L=selectedBoxes.size();
		for (i=0;i<L;i++) {
			g=(GraphBox)selectedBoxes.get(i);
			g.addTransitionTo(dest);
		}	
	}


	public void addReverseTransitions(GraphBox from) {
		int i,L;
		GraphBox g;
		if (selectedBoxes.isEmpty()) return;
		L=selectedBoxes.size();
		for (i=0;i<L;i++) {
			g=(GraphBox)selectedBoxes.get(i);
			from.addTransitionTo(g);
		}	
	}


	public void drawAllTransitions(Graphics gr) {
		int i,L;
		GraphBox g;
		if (graphBoxes.isEmpty()) return;
		L=graphBoxes.size();
		for (i=0;i<L;i++) {
			g=(GraphBox)graphBoxes.get(i);
			g.drawTransitions(gr);
		}	
	}


	public void drawTransitionsFromMousePointerToSelectedBoxes(Graphics gr) {
		GraphBox temp=new GraphBox(Xmouse,Ymouse,2,gr,this);
		GraphBox g;
		temp.X_out=Xmouse;
		temp.Y_out=Ymouse;
		int L=selectedBoxes.size();
		for (int i=0;i<L;i++) {
			g=(GraphBox)selectedBoxes.get(i);
			temp.drawTransition(gr,g);
		}	
	}


	public void drawTransitionsFromSelectedBoxesToMousePointer(Graphics gr) {
		GraphBox temp=new GraphBox(Xmouse,Ymouse,2,gr,this);
		GraphBox g;
		temp.X_out=Xmouse;
		temp.Y_out=Ymouse;
		int L=selectedBoxes.size();
		for (int i=0;i<L;i++) {
			g=(GraphBox)selectedBoxes.get(i);
			g.drawTransition(gr,temp);
		}	
	}

	
	public void drawAllBoxes(Graphics gr) {
		int i,L;
		GraphBox g;
		if (graphBoxes.isEmpty()) return;
		L=graphBoxes.size();
		for (i=0;i<L;i++) {
			g=(GraphBox)graphBoxes.get(i);
			g.draw(gr);
		}	
	}


	public void drawGrid(Graphics f) {
		if (!isGrid) return;
		int x,y;
		f.setColor(pref.foregroundColor);
		for (x=10;x<Width-10;x=x+nPixels)
			for (y=10;y<Height-10;y=y+nPixels)
				f.drawLine(x,y,x+1,y);
	}


	public void paintComponent(Graphics f_old) {
		Graphics2D f=(Graphics2D)f_old;
		f.scale(scaleFactor,scaleFactor);
		if (!is_initialised) {
			this.init();
			is_initialised=true;
		}
		if (pref.antialiasing) {
			f.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
		}
		else {
			f.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_OFF);
		}
		f.setColor(new Color(205,205,205));
		f.fillRect(0,0,getWidth(),getHeight());
		f.setColor(pref.backgroundColor);
		f.fillRect(0,0,Width,Height);
		if (pref.frame) {
			f.setColor(pref.foregroundColor);
			f.drawRect(10,10,Width-20,Height-20);
			f.drawRect(9,9,Width-18,Height-18);
		}
		f.setColor(pref.foregroundColor);
		if (pref.filename) {
			if (pref.pathname)
				f.drawString(parent().pathname+"\\"+parent().filename,20,Height-45);
			else f.drawString(parent().filename,20,Height-45);
		}
		if (pref.date)
			f.drawString(new Date().toString(),20,Height-25);
		drawGrid(f);
		if (mouseInGraphicalZone && !selectedBoxes.isEmpty()) {
			if (EDITING_MODE==MyCursors.REVERSE_LINK_BOXES) {
				drawTransitionsFromMousePointerToSelectedBoxes(f);
			}
			else if (EDITING_MODE==MyCursors.LINK_BOXES) {
				drawTransitionsFromSelectedBoxesToMousePointer(f);
			}
		}
		drawAllTransitions(f);
		drawAllBoxes(f);
		if (selecting) {
			// here we draw the selection rectangle
			f.setColor(pref.foregroundColor);
			f.drawRect(X_drag,Y_drag,dragWidth,dragHeight);
		}
	}



/*
 * Box alignment methods
 *
 */


	public void HTopAlign() {
		int y1,i;
		GraphBox g;
		if (selectedBoxes.isEmpty()) return;
		g=(GraphBox)selectedBoxes.get(0);
		y1=g.Y1;
		for (i=1;i<selectedBoxes.size();i++) {
			g=(GraphBox)selectedBoxes.get(i);
			if (g.Y1<y1) y1=g.Y1;
		}
		// now, y1 is the value that will be common to the selected boxes
		for (i=0;i<selectedBoxes.size();i++) {
			g=(GraphBox)selectedBoxes.get(i);
			int dy=y1-g.Y1;
			g.translate(0,dy);
		}
		repaint();
	}


	public void HCenterAlign() {
		if (selectedBoxes.isEmpty()) return;
		int y1,i;
		GraphBox g;
		if (selectedBoxes.isEmpty()) return;
		y1=0;
		for (i=0;i<selectedBoxes.size();i++) {
			g=(GraphBox)selectedBoxes.get(i);
			y1=y1+g.Y;
		}
		y1=y1/selectedBoxes.size();
		// now, y1 is the value that will be common to the selected boxes
		for (i=0;i<selectedBoxes.size();i++) {
			g=(GraphBox)selectedBoxes.get(i);
			int dy=y1-g.Y;
			g.translate(0,dy);
		}
		repaint();
		
	}


	public void HBottomAlign() {
		int y1,i;
		GraphBox g;
		if (selectedBoxes.isEmpty()) return;
		g=(GraphBox)selectedBoxes.get(0);
		y1=g.Y1+g.Height;
		for (i=1;i<selectedBoxes.size();i++) {
			g=(GraphBox)selectedBoxes.get(i);
			if ((g.Y1+g.Height)>y1) y1=g.Y1+g.Height;
		}
		// now, y1 is the value that will be common to the selected boxes
		for (i=0;i<selectedBoxes.size();i++) {
			g=(GraphBox)selectedBoxes.get(i);
			int dy=y1-(g.Y1+g.Height);
			g.translate(0,dy);
		}
		repaint();
	}


	public void VLeftAlign() {
		int x1,i;
		GraphBox g;
		if (selectedBoxes.isEmpty()) return;
		g=(GraphBox)selectedBoxes.get(0);
		x1=g.X1;
		for (i=1;i<selectedBoxes.size();i++) {
			g=(GraphBox)selectedBoxes.get(i);
			if (g.X1<x1) x1=g.X1;
		}
		// now, x1 is the value that will be common to the selected boxes
		for (i=0;i<selectedBoxes.size();i++) {
			g=(GraphBox)selectedBoxes.get(i);
			int dx=x1-g.X1;
			g.translate(dx,0);
		}
		repaint();
	}


	public void VCenterAlign() {
		int x1,i;
		GraphBox g;
		if (selectedBoxes.isEmpty()) return;
		g=(GraphBox)selectedBoxes.get(0);
		x1=0;
		for (i=0;i<selectedBoxes.size();i++) {
			g=(GraphBox)selectedBoxes.get(i);
			x1=x1+(g.X1+g.Width/2);
		}
		x1=x1/selectedBoxes.size();
		// now, x1 is the value that will be common to the selected boxes
		for (i=0;i<selectedBoxes.size();i++) {
			g=(GraphBox)selectedBoxes.get(i);
			int dx=x1-(g.X1+g.Width/2);
			g.translate(dx,0);
		}
		repaint();
	}


	public void VRightAlign() {
		int x1,i;
		GraphBox g;
		if (selectedBoxes.isEmpty()) return;
		g=(GraphBox)selectedBoxes.get(0);
		x1=g.X1+g.Width;
		for (i=1;i<selectedBoxes.size();i++) {
			g=(GraphBox)selectedBoxes.get(i);
			if ((g.X1+g.Width)>x1) x1=g.X1+g.Width;
		}
		// now, x1 is the value that will be common to the selected boxes
		for (i=0;i<selectedBoxes.size();i++) {
			g=(GraphBox)selectedBoxes.get(i);
			int dx=x1-(g.X1+g.Width);
			g.translate(dx,0);
		}
		repaint();
	}


	public void setGrid(boolean b) {
		isGrid=b;
		repaint();
	}


	public void setGrid(boolean b,int n) {
		isGrid=b;
		nPixels=n;
		repaint();
	}


	public void updateAllBoxes() {
		GraphBox g;
		int i;
		for (i=0;i<graphBoxes.size();i++) {
			g=(GraphBox)graphBoxes.get(i);
			g.update();
		}
		repaint();
	}


	public void setRightToLeft(boolean b) {
		pref.rightToLeft=b;
	}



/*
 * This is the print method
 *
 */
	public int print(Graphics g,PageFormat p,int pageIndex) {
		if (pageIndex!=0) return Printable.NO_SUCH_PAGE;

		Graphics2D f=(Graphics2D)g;
		double DPI=96.0;//(double)Toolkit.getDefaultToolkit().getScreenResolution();
		double WidthInInches=p.getImageableWidth()/72;
		double realWidthInInches=(Width/DPI);
		double HeightInInches=p.getImageableHeight()/72;
		double realHeightInInches=(Height/DPI);
		
		double scale_x=WidthInInches/realWidthInInches;
		double scale_y=HeightInInches/realHeightInInches;
		
		f.translate(p.getImageableX(),p.getImageableY());
		if (scale_x<scale_y)
			f.scale(0.99*0.72*scale_x,0.99*0.72*scale_x);
		else f.scale(0.99*0.72*scale_y,0.99*0.72*scale_y);
		f.setColor(pref.backgroundColor);
		f.fillRect(0,0,Width,Height);
		if (pref.frame) {
			f.setColor(pref.foregroundColor);
			f.drawRect(10,10,Width-20,Height-20);
			f.drawRect(9,9,Width-18,Height-18);
		}
		f.setColor(pref.foregroundColor);
		if (pref.filename) {
			if (pref.pathname)
				f.drawString(parent().pathname+"\\"+parent().filename,20,Height-45);
			else f.drawString(parent().filename,20,Height-45);
		}
		if (pref.date)
			f.drawString(new Date().toString(),20,Height-25);
		drawGrid(f);
		drawAllTransitions(f);
		drawAllBoxes(f);
		if (selecting) {
			// here we draw the selection rectangle
			f.drawRect(X_drag,Y_drag,dragWidth,dragHeight);
		}
		return Printable.PAGE_EXISTS;
	}







} /* end of GraphicalZone */
