 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.io.* ;

/*
 * This class describes the lexicon-grammar conversion window
 *
 */
public class LexiconGrammarFrame extends JInternalFrame {
	
	public static LexiconGrammarFrame frame;
	public JPanel panel=new JPanel();
	public JPanel upPanel=new JPanel();
	public JPanel downPanel=new JPanel();
	public JTextField grfName=new JTextField();
	public JButton setGrfName=new JButton("Set...");
	public JTextField resultName=new JTextField();
	public JButton setResultName=new JButton("Set...");
	public JButton CANCEL=new JButton("Cancel");
	public JButton OK=new JButton("Compile");
	public String nombre_matches=null;
	
	
	public LexiconGrammarFrame() {
		super("Compile Lexicon-Grammar to GRF",false,true);
		constructPanel();
		setContentPane(panel);
		pack();
		setResizable(false);
		setVisible(false);
		addInternalFrameListener(new InternalFrameListener() {
			public void internalFrameActivated(InternalFrameEvent e) {}
			public void internalFrameClosed(InternalFrameEvent e) {}
			public void internalFrameDeactivated(InternalFrameEvent e) {}
			public void internalFrameDeiconified(InternalFrameEvent e) {}
			public void internalFrameIconified(InternalFrameEvent e) {}
			public void internalFrameOpened(InternalFrameEvent e) {}
			public void internalFrameClosing(InternalFrameEvent e) {
				setVisible(false);
			}
		});
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	}


	public static void init() {
		frame=new LexiconGrammarFrame();
	}
	
	
	public static LexiconGrammarFrame getFrame() {
		return frame;
	}


	public static void showFrame() {
		frame.setVisible(true);
		try { 
		    frame.setSelected(true); 
		    frame.setIcon(false);
		} catch (java.beans.PropertyVetoException e2) {}
	}
	

	
	public void constructPanel() {
		panel.setOpaque(true);
		panel.setLayout(new BorderLayout());
		constructUpPanel();
		constructDownPanel();
		panel.add(upPanel,BorderLayout.NORTH);
		panel.add(downPanel,BorderLayout.CENTER);
	}



	public JPanel createPanel(JLabel label,JTextField textField,JButton button) {
		JPanel p=new JPanel();
		p.setOpaque(true);
		p.setLayout(new GridLayout(2,1));
		p.add(label);
		JPanel tmp=new JPanel();
		tmp.setOpaque(true);
		tmp.setLayout(new BorderLayout());
		tmp.add(textField,BorderLayout.CENTER);
		tmp.add(button,BorderLayout.EAST);
		p.add(tmp);
		return p;
	}
	
	

	public void constructUpPanel() {
		upPanel.setBorder(new EmptyBorder(5,5,5,5));
		upPanel.setLayout(new GridLayout(2,1));
		grfName.setPreferredSize(new Dimension(280,20));
		resultName.setPreferredSize(new Dimension(280,20));
		setGrfName.addMouseListener(new SetGrfNameButtonListener());
		setResultName.addMouseListener(new SetResultNameButtonListener());
		JPanel a=createPanel(new JLabel("Reference Graph (in GRF format):"),grfName,setGrfName);
		JPanel b=createPanel(new JLabel("Resulting GRF grammar:"),resultName,setResultName);
		upPanel.add(a);
		upPanel.add(b);
	}


	public void constructDownPanel() {
		downPanel.setOpaque(true);
		downPanel.setLayout(new GridLayout(1,2));
		OK.addMouseListener(new OKButtonListener());
		CANCEL.addMouseListener(new CANCELButtonListener());
		JPanel left=new JPanel();
		left.setBorder(new EmptyBorder(10,50,10,20));
		left.setLayout(new BorderLayout());
		left.add(CANCEL,BorderLayout.CENTER);
		JPanel right=new JPanel();
		right.setBorder(new EmptyBorder(10,20,10,50));
		right.setLayout(new BorderLayout());
		right.add(OK,BorderLayout.CENTER);
		downPanel.add(left);
		downPanel.add(right);
	}





class SetGrfNameButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		JFileChooser chooser=new JFileChooser();
		chooser.addChoosableFileFilter(new PersonalFileFilter("grf","Unicode Graphs"));
		chooser.setDialogType(JFileChooser.OPEN_DIALOG);
		chooser.setCurrentDirectory(new File(Config.getCurrentGraphDir()));
		int returnVal=chooser.showOpenDialog(null);
		if (returnVal!=JFileChooser.APPROVE_OPTION) {
			// we return if the user has clicked on CANCEL
			return;
		}
		frame.grfName.setText(chooser.getSelectedFile().getAbsolutePath());
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}



class SetResultNameButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		JFileChooser chooser=new JFileChooser();
		chooser.addChoosableFileFilter(new PersonalFileFilter("grf","Unicode Graphs"));
		chooser.setDialogType(JFileChooser.SAVE_DIALOG);
		chooser.setCurrentDirectory(new File(Config.getCurrentGraphDir()));
		int returnVal=chooser.showOpenDialog(null);
		if (returnVal!=JFileChooser.APPROVE_OPTION) {
			// we return if the user has clicked on CANCEL
			return;
		}
		String s=chooser.getSelectedFile().getAbsolutePath();
		if (!s.endsWith(".grf")) s=s+".grf";
		frame.resultName.setText(s);
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}



class OKButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				String command;
				command="\""+Config.getApplicationDir()+"Table2Grf\" \"";
				command=command+LexiconGrammarTableFrame.tableName+"\" \"";
				command=command+frame.grfName.getText()+"\" \"";
				command=command+frame.resultName.getText()+"\"";
				frame.setVisible(false);
				new ProcessInfoFrame(command,true,null);
			}
		});
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class CANCELButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		frame.setVisible(false);
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


}
