 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import java.io.* ;

/*
 * This class describes the preprocessing window
 *
 */
public class PreprocessFrame extends JDialog {
	
	public JPanel panel=new JPanel();
	public JPanel processing=new JPanel();
	public JPanel lexicalParsing=new JPanel();
	public JPanel tokenizingPanel=new JPanel();
	public JPanel buttons=new JPanel();
	public JPanel sentence=new JPanel();
	public JPanel replace=new JPanel();
	public JButton OK=new JButton("GO!");
	public JButton CANCELbutIndex=new JButton("Cancel but tokenize text");
	public JButton CANCEL=new JButton("Cancel and close text");
	public JCheckBox sentenceCheck=new JCheckBox("Apply FST2 in MERGE mode:",true);
	public JCheckBox replaceCheck=new JCheckBox("Apply FST2 in REPLACE mode:",true);
	public JTextField sentenceName=new JTextField(Config.getCurrentSentenceGraph());
	public JTextField replaceName=new JTextField(Config.getCurrentReplaceGraph());
	public JButton setSentence=new JButton("Set...");
	public JButton setReplace=new JButton("Set...");
	public JCheckBox applyDicCheck=new JCheckBox("Apply All default Dictionaries",true);
	public JCheckBox textFst2Check=new JCheckBox("Construct Text Automaton",false);
	public String name;
	String snt;
	

	public PreprocessFrame(String n,String snt_name) {
		super(UnitexFrame.mainFrame,"Preprocessing & Lexical parsing",true);
		name=n;
		snt=snt_name;
		constructPanel();
		setContentPane(panel);
		pack();
		setResizable(false);
		setLocationRelativeTo(UnitexFrame.mainFrame);
		this.setVisible(true);
	}
	
	
	public void constructPanel() {
		//panel.setLayout(new GridLayout(2,1));
		panel.setLayout(new BorderLayout());
		constructProcessingPanel();
		constructLexicalParsingPanel();
		constructButtonsPanel();
		constructTokenizingPanel();
		panel.add(processing,BorderLayout.NORTH);
		panel.add(tokenizingPanel,BorderLayout.CENTER);
		JPanel down=new JPanel();
		down.setLayout(new GridLayout(1,2));
		down.add(lexicalParsing);
		down.add(buttons);
		panel.add(down,BorderLayout.SOUTH);
	}
	
	
	public void constructProcessingPanel() {
		processing.setBorder(new TitledBorder("Preprocessing"));
		processing.setLayout(new GridLayout(2,1));
		constructSentencePanel();
		constructReplacePanel();
		processing.add(sentence);
		processing.add(replace);
	}


	public void constructSentencePanel() {
		sentence.setLayout(new GridBagLayout());
		Dimension d=sentenceCheck.getPreferredSize();
		sentenceCheck.setPreferredSize(new Dimension(200,d.height));
		sentence.add(sentenceCheck);
		d=sentenceName.getPreferredSize();
		sentenceName.setPreferredSize(new Dimension(200,d.height));
		sentence.add(sentenceName);
		setSentence.addMouseListener(new SetSentenceListener());
		sentence.add(setSentence);
	}
	

	public void constructReplacePanel() {
		replace.setLayout(new GridBagLayout());
		Dimension d=replaceCheck.getPreferredSize();
		replaceCheck.setPreferredSize(new Dimension(200,d.height));
		replace.add(replaceCheck);
		d=replaceName.getPreferredSize();
		replaceName.setPreferredSize(new Dimension(200,d.height));
		replace.add(replaceName);
		setReplace.addMouseListener(new SetReplaceListener());
		replace.add(setReplace);
	}

	
	public void constructTokenizingPanel() {
		tokenizingPanel.setBorder(new TitledBorder("Tokenizing"));
		tokenizingPanel.setLayout(new GridLayout(2,1));
		tokenizingPanel.add(new JLabel("The text is automatically tokenized. This operation is language-dependant,"));
		tokenizingPanel.add(new JLabel("so that Unitex can handle languages with special spacing rules."));
	}


	public void constructLexicalParsingPanel() {
		lexicalParsing.setBorder(new TitledBorder("Lexical Parsing"));
		lexicalParsing.setLayout(new GridLayout(2,1));
		lexicalParsing.add(applyDicCheck);
		lexicalParsing.add(textFst2Check);
	}
	

	public void constructApplyDicPanel() {
	}


	public void constructTextFst2Panel() {
	}

	
	public void constructButtonsPanel() {
		buttons.setBorder(new EmptyBorder(8,8,2,2));
		buttons.setLayout(new GridLayout(3,1));
		OK.addMouseListener(new OKListener());
		CANCELbutIndex.addMouseListener(new CANCELbutIndexListener());
		CANCEL.addMouseListener(new CANCELListener());
		buttons.add(OK);
		buttons.add(CANCELbutIndex);
		buttons.add(CANCEL);
	}







class SetSentenceListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		JFileChooser chooser=new JFileChooser();
		chooser.addChoosableFileFilter(new PersonalFileFilter("fst2","Unicode Compiled Graphs"));
		chooser.setDialogType(JFileChooser.OPEN_DIALOG);
		chooser.setCurrentDirectory(new File(Config.getUserCurrentLanguageDir()+"Graphs"+File.separatorChar+"Preprocessing"+File.separatorChar+"Sentence"));
		int returnVal=chooser.showOpenDialog(null);
		if (returnVal!=JFileChooser.APPROVE_OPTION) {
			// we return if the user has clicked on CANCEL
			return;
		}
		sentenceName.setText(chooser.getSelectedFile().getAbsolutePath());
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}



class SetReplaceListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		JFileChooser chooser=new JFileChooser();
		chooser.addChoosableFileFilter(new PersonalFileFilter("fst2","Unicode Compiled Graphs"));
		chooser.setDialogType(JFileChooser.OPEN_DIALOG);
		chooser.setCurrentDirectory(new File(Config.getUserCurrentLanguageDir()+"Graphs"+File.separatorChar+"Preprocessing"+File.separatorChar+"Replace"));
		int returnVal=chooser.showOpenDialog(null);
		if (returnVal!=JFileChooser.APPROVE_OPTION) {
			// we return if the user has clicked on CANCEL
			return;
		}
		replaceName.setText(chooser.getSelectedFile().getAbsolutePath());
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class OKListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		e.consume();
		setVisible(false);
		SwingUtilities.invokeLater(new Runnable() {
			public void run(){
				File dir=new File(Config.getCurrentSntDir());
				if (!dir.exists()) {
					// if the directory toto_snt does not exist, we create it
					dir.mkdir();
				}
				String command[]=new String[9];
				// NORMALIZING TEXT...
				command[0]="\""+Config.getApplicationDir()+"Normalize\" \""+name+"\"";
				// SENTENCE GRAPH...
				if (sentenceCheck.isSelected() && new File(sentenceName.getText()).exists()) {
					command[1]="\""+Config.getApplicationDir()+"Fst2Txt\" \""+Config.getCurrentSnt()+"\" ";
					command[1]=command[1]+"\""+sentenceName.getText()+"\" ";
					command[1]=command[1]+"\""+Config.getAlphabet()+"\" ";
					command[1]=command[1]+"-merge ";
					if (Config.getCurrentLanguage().equals("Thai") ||
					   	Config.getCurrentLanguage().equals("Korean")) {
						command[1]=command[1]+"-char_by_char";
					}
				}
				// REPLACE GRAPH...
				if (replaceCheck.isSelected() && new File(replaceName.getText()).exists()) {
					command[2]="\""+Config.getApplicationDir()+"Fst2Txt\" \""+Config.getCurrentSnt()+"\" ";
					command[2]=command[2]+"\""+replaceName.getText()+"\" ";
					command[2]=command[2]+"\""+Config.getAlphabet()+"\" ";
					command[2]=command[2]+"-replace ";
					if (Config.getCurrentLanguage().equals("Thai") ||
					   	Config.getCurrentLanguage().equals("Korean")) {
						command[2]=command[2]+"-char_by_char";
					}
				}
				// TOKENIZING...
				command[3]="\""+Config.getApplicationDir()+"Tokenize\" \""+Config.getCurrentSnt()+"\"";
				command[3]=command[3]+" \""+Config.getAlphabet()+"\"";
				if (Config.getCurrentLanguage().equals("Thai") ||
				    Config.getCurrentLanguage().equals("Korean")) {
						command[3]=command[3]+" -char_by_char";
				}
				// APPLYING DEFAULT DICTIONARIES...
				if (applyDicCheck.isSelected()) {
					command[4]="\""+Config.getApplicationDir()+"Dico\" \""+Config.getCurrentSnt()+"\" ";
					command[4]=command[4]+"\""+Config.getAlphabet()+"\" ";
					String param=getDefaultDicList();
					if (param!=null && !param.equals("")) {
						command[4]=command[4]+param;
					}
					else {
						command[4]=null;
					}
					// then, sorting result dictionaries
					if (command[4]!=null) {
						// DLF
						command[5]="\""+Config.getApplicationDir()+"SortTxt\" \""+Config.getCurrentSntDir();
						command[5]=command[5]+"dlf\" -y";
						command[5]=command[5]+" -l \""+Config.getCurrentSntDir()+"dlf.n\"";
						if (Config.getCurrentLanguage().equals("Thai") ||
						   	Config.getCurrentLanguage().equals("Korean")) {
							command[5]=command[5]+" -thai";
						} else {
							command[5]=command[5]+" -o \""+Config.getUserCurrentLanguageDir()+"Alphabet_sort.txt\"";
						}
						//  DLC
						command[6]="\""+Config.getApplicationDir()+"SortTxt\" \""+Config.getCurrentSntDir();
						command[6]=command[6]+"dlc\" -y";
						command[6]=command[6]+" -l \""+Config.getCurrentSntDir()+"dlc.n\"";
						if (Config.getCurrentLanguage().equals("Thai") ||
						   	Config.getCurrentLanguage().equals("Korean")) {
								command[6]=command[6]+" -thai";
						} else {
							command[6]=command[6]+" -o \""+Config.getUserCurrentLanguageDir()+"Alphabet_sort.txt\"";
						}
						//  and ERR
						command[7]="\""+Config.getApplicationDir()+"SortTxt\" \""+Config.getCurrentSntDir();
						command[7]=command[7]+"err\" -y";
						command[7]=command[7]+" -l \""+Config.getCurrentSntDir()+"err.n\"";
						if (Config.getCurrentLanguage().equals("Thai") ||
						   	Config.getCurrentLanguage().equals("Korean")) {
								command[7]=command[7]+" -thai";
						} else {
							command[7]=command[7]+" -o \""+Config.getUserCurrentLanguageDir()+"Alphabet_sort.txt\"";
						}
					}
				}
				// CONSTRUCTING TEXT AUTOMATON
				if (textFst2Check.isSelected()) {
					command[8]="\""+Config.getApplicationDir()+"Txt2Fst2\" \""+Config.getCurrentSnt()+"\" ";
					command[8]=command[8]+"\""+Config.getAlphabet()+"\" -clean";
				}
				UnitexFrame.mainFrame.closeText();
				Text.removeSntFiles();
				new ProcessInfoFrame(command,true,new PreprocessDo(snt));
			}
		});
		dispose();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class CANCELbutIndexListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		// if the user has clicked on CANCEL but tokenize, we must
		// tokenize anyway
		setVisible(false);
		e.consume();
		SwingUtilities.invokeLater(new Runnable() {
			public void run(){
				File dir=new File(Config.getCurrentSntDir());
				if (!dir.exists()) {
					// if the directory toto_snt does not exist, we create it
					dir.mkdir();
				}
				String command[]=new String[2];
				// NORMALIZING TEXT...
				command[0]="\""+Config.getApplicationDir()+"Normalize\" \""+name+"\"";
				// TOKENIZING...
				command[1]="\""+Config.getApplicationDir()+"Tokenize\" \""+Config.getCurrentSnt()+"\"";
				command[1]=command[1]+" \""+Config.getAlphabet()+"\"";
				if (Config.getCurrentLanguage().equals("Thai") ||
				    Config.getCurrentLanguage().equals("Korean")) {
						command[1]=command[1]+" -char_by_char";
				}
				UnitexFrame.mainFrame.closeText();
				Text.removeSntFiles();
				new ProcessInfoFrame(command,true,new PreprocessDo(snt));
			}
		});
		dispose();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}



class CANCELListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		// if the user has clicked on CANCEL, we do nothing
		setVisible(false);
		e.consume();
		UnitexFrame.mainFrame.closeText();
		dispose();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


	public String getDefaultDicList() {
		String res="";
		String name;
		name=Config.getUserCurrentLanguageDir()+"user_dic.def";
		try {
			BufferedReader br=new BufferedReader(new FileReader(name));
			String s;
			while ((s=br.readLine())!=null) {
				res=res+" \"";
				res=res+Config.getUserCurrentLanguageDir()+"Dela"+File.separator+s+"\"";
			}
			br.close();
		} catch (IOException e) {}
		name=Config.getUserCurrentLanguageDir()+"system_dic.def";
		try {
			BufferedReader br=new BufferedReader(new FileReader(name));
			String s;
			while ((s=br.readLine())!=null) {
				res=res+" \"";
				res=res+Config.getUnitexCurrentLanguageDir()+"Dela"+File.separator+s+"\"";
			}
			br.close();
		} catch (IOException e) {}
		return res;
	}


class PreprocessDo extends DoAbstract {
	String SNT;

	public PreprocessDo(String s) {
		SNT=s;
	}
	public void DO() {
		Text.loadSnt(SNT);
	}	
}

}
