 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import java.io.* ;

/*
 * This class provides methods for loading corpora
 *
 */
public class Text {
	
	public static void loadCorpus(String name) {
		if (getExtension(new File(name)).equalsIgnoreCase("snt")) {
			Config.setCurrentSnt(name);
			loadSnt(name);
		}
		else {
			loadTxt(name);
		}
	}
	

	public static void loadTxt(String name) {
		File infile;
		FileInputStream source;
		infile=new File(name);
		if (!infile.exists()) {
			JOptionPane.showMessageDialog(null,"Cannot find "+name,"Error",JOptionPane.ERROR_MESSAGE); 
			return;
		}
		if (!infile.canRead()) {
			JOptionPane.showMessageDialog(null,"Cannot read "+name,"Error",JOptionPane.ERROR_MESSAGE); 
			return;
		}
		if (infile.length()<=2) {
			JOptionPane.showMessageDialog(null,name+" is empty","Error",JOptionPane.ERROR_MESSAGE); 
			return;
		}
		try {
			source=new FileInputStream(infile);
			char c=GraphIO.read_char(source);
			source.close();
			if (c!=0xFEFF) {
				JOptionPane.showMessageDialog(null,name+" is not a Unicode Little-Endian text","Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
		} catch (IOException e) {};
		String nomSnt=name.substring(0,name.length()-3);
		nomSnt=nomSnt+"snt";
		Config.setCurrentSnt(nomSnt);
		Object[] options = {"Yes","No"};
		if (0==JOptionPane.showOptionDialog(null,"Do you want to preprocess the text ?",""
		    ,JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE,null,options,options[0])) {
	    	preprocessSnt(name,Config.getCurrentSnt());
		}
		else {
	    	preprocessLightSnt(name,Config.getCurrentSnt());
		}
	}


	public static void preprocessSnt(String name,String snt) {
		new PreprocessFrame(name,snt);
	}


	public static void preprocessLightSnt(String name,String snt) {
		File dir=new File(Config.getCurrentSntDir());
		if (!dir.exists()) {
			// if the directory toto_snt does not exist, we create it
			dir.mkdir();
		}
		String command[]=new String[2];
		// NORMALIZING TEXT...
		command[0]="\""+Config.getApplicationDir()+"Normalize\" \""+name+"\"";
		// TOKENIZING...
		command[1]="\""+Config.getApplicationDir()+"Tokenize\" \""+Config.getCurrentSnt()+"\"";
		command[1]=command[1]+" \""+Config.getAlphabet()+"\"";
		if (Config.getCurrentLanguage().equals("Thai") ||
		    Config.getCurrentLanguage().equals("Korean")) {
				command[1]=command[1]+" -char_by_char";
		}
		UnitexFrame.mainFrame.closeText();
		Text.removeSntFiles();
		new ProcessInfoFrame(command,true,new TextDo(Config.getCurrentSnt()));
	}


	public static String getExtension(File f) {
		String ext=null;
		String s=f.getName();
		int i=s.lastIndexOf('.');
		if (i>0 && i<s.length()-1) {
			ext = s.substring(i+1).toLowerCase();
		}
		return ext;
	}


	public static void loadSnt(String snt) {
		UnitexFrame.mainFrame.closeText();
		TextFrame.loadText(snt);
		TokensFrame.loadTokens(Config.getCurrentSntDir()+"tok_by_freq.txt");
		TextDicFrame.loadTextDic(Config.getCurrentSntDir());
		TextAutomatonFrame.showFrame();
		try {
			TextFrame.getFrame().setSelected(true);
		} catch(java.beans.PropertyVetoException e) {}
	}
	
	
	public static void closeText() {
		TextFrame.closeText();
	}
	

	public static void removeSntFiles() {
		Config.removeFile(Config.getCurrentSntDir()+"*");
	}
	
	

static class TextDo extends DoAbstract {
	String SNT;

	public TextDo(String s) {
		SNT=s;
	}
	public void DO() {
		Text.loadSnt(SNT);
	}	
}

}