 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Hashtable;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.border.*;

/*
 * This class displays the text font selection window
 *
 */

public class TextFontMenu extends JDialog {
	
	JPanel panel=new JPanel();
	JPanel upPanel=new JPanel();
	JPanel downPanel=new JPanel();
	JPanel fontPanel=new JPanel();
	JPanel stylePanel=new JPanel();
	JPanel sizePanel=new JPanel();
	JPanel scriptPanel=new JPanel();
	JPanel exampleAndButtonPanel=new JPanel();
	JPanel examplePanel=new JPanel();
	JPanel buttonPanel=new JPanel();
	
	JLabel fontLabel=new JLabel(" Font: ");
	JLabel styleLabel=new JLabel(" Style: ");
	JLabel sizeLabel=new JLabel(" Size: ");
    JLabel scriptLabel=new JLabel(" Script: ");
    JTextField font=new JTextField(10);
    JTextField style=new JTextField(10);
    NumericTextField size=new NumericTextField(3,"");
    JTextField script=new JTextField(10);
    JTextField example=new JTextField(6);
    JButton OK=new JButton("OK");
    JButton CANCEL=new JButton("Cancel");
    FontList fontList;
    StyleList styleList;
    SizeList sizeList;
    ScriptList scriptList;
    Hashtable styles;
    Hashtable ranges;

    static final int ASCII_BASE    = 0x0000;
    static final int GREEK_BASE    = 0x0370;
    static final int HEBREW_BASE   = 0x0590;
    static final int ARABIC_BASE   = 0x0600;
    static final int THAI_BASE     = 0x0E00;
    static final int GEORGIAN_BASE = 0x10A0;
    static final int HIRAGANA_BASE = 0x3040;
    static final int KATAKANA_BASE = 0x30A0;
    static final int HANGUL_BASE   = 0x3130;
    static final int KANJI_BASE    = 0x4E00;

	String fontName;
	int fontStyle;
	int fontSize;
	int fontScript;

	Font res;


	public TextFontMenu() {
		super(UnitexFrame.mainFrame,"Text Font",true);
		fontName=Preferences.temp.textFont.getName();
		fontStyle=Preferences.temp.textFont.getStyle();
		fontSize=Preferences.temp.textFontSize;
		init();
		constructPanel();
		refresh();
		setContentPane(panel);
		pack();
		setResizable(false);
		showFontMenu();	
	}


	public void showFontMenu() {
		setLocationRelativeTo(UnitexFrame.mainFrame);
		this.setVisible(true);
	}


	public void constructPanel() {
		panel.setLayout(new BorderLayout());
		constructUpPanel();
		constructDownPanel();
		panel.add(upPanel,BorderLayout.CENTER);
		panel.add(downPanel,BorderLayout.SOUTH);
	}


	public void constructUpPanel() {
		upPanel.setLayout(new BorderLayout());
		constructFontPanel();
		constructStylePanel();
		constructSizePanel();
		upPanel.add(fontPanel,BorderLayout.WEST);
		upPanel.add(stylePanel,BorderLayout.CENTER);
		upPanel.add(sizePanel,BorderLayout.EAST);
	}


	public void constructDownPanel() {
		downPanel.setLayout(new BorderLayout());
		constructScriptPanel();
		constructExampleAndButtonPanel();
		downPanel.add(scriptPanel,BorderLayout.WEST);
		downPanel.add(exampleAndButtonPanel,BorderLayout.CENTER);
	}


	public void constructFontPanel() {
		fontPanel.setBorder(new EmptyBorder(5,5,5,5));
		fontPanel.setLayout(new BorderLayout());
		JPanel p=new JPanel();
		p.setLayout(new GridLayout(2,1));
		p.add(fontLabel);
		p.add(font);
		fontPanel.add(p,BorderLayout.NORTH);
		fontPanel.add(fontList,BorderLayout.CENTER);
	}


	public void constructStylePanel() {
		stylePanel.setBorder(new EmptyBorder(5,5,5,5));
		stylePanel.setLayout(new BorderLayout());
		JPanel p=new JPanel();
		p.setLayout(new GridLayout(2,1));
		p.add(styleLabel);
		p.add(style);
		stylePanel.add(p,BorderLayout.NORTH);
		stylePanel.add(styleList,BorderLayout.CENTER);
	}


	public void constructSizePanel() {
		sizePanel.setBorder(new EmptyBorder(5,5,5,5));
		sizePanel.setLayout(new BorderLayout());
		JPanel p=new JPanel();
		p.setLayout(new GridLayout(2,1));
		p.add(sizeLabel);
		p.add(size);
		sizePanel.add(p,BorderLayout.NORTH);
		sizePanel.add(sizeList,BorderLayout.CENTER);
	}


	public void constructScriptPanel() {
		scriptPanel.setBorder(new EmptyBorder(5,5,5,5));
		scriptPanel.setLayout(new BorderLayout());
		JPanel p=new JPanel();
		p.setLayout(new GridLayout(2,1));
		p.add(scriptLabel);
		p.add(script);
		scriptPanel.add(p,BorderLayout.NORTH);
		scriptPanel.add(scriptList,BorderLayout.CENTER);
	}


	public void constructExampleAndButtonPanel() {
		constructExamplePanel();
		constructButtonPanel();
		exampleAndButtonPanel.setBorder(new EmptyBorder(0,0,0,5));
		exampleAndButtonPanel.setLayout(new BorderLayout());
		exampleAndButtonPanel.add(examplePanel,BorderLayout.CENTER);
		exampleAndButtonPanel.add(buttonPanel,BorderLayout.SOUTH);
	}
	
	
	public void constructExamplePanel() {
		examplePanel.setBorder(new TitledBorder("Example"));
		examplePanel.setLayout(new BorderLayout());
		examplePanel.add(new JPanel(),BorderLayout.WEST);
		examplePanel.add(example,BorderLayout.CENTER);
		examplePanel.add(new JPanel(),BorderLayout.EAST);
	}


	public void constructButtonPanel() {
		buttonPanel.setLayout(new FlowLayout());
		OK.addMouseListener(new OKListener());
		CANCEL.addMouseListener(new CANCELListener());
		buttonPanel.add(OK);
		buttonPanel.add(CANCEL);
	}


	public void init() {
		// setting the components form
		fontList=new FontList(7);
		font.setBackground(Color.white);
		font.setText("");
		styleList=new StyleList(7);
		style.setDisabledTextColor(Color.white);
		style.setBackground(Color.white);
		style.setText("");
		style.setEditable(false);
		sizeList=new SizeList(7);
		size.setDisabledTextColor(Color.white);
		size.setBackground(Color.white);
		size.setText("");
		size.setEditable(false);
		scriptList=new ScriptList(5);
		script.setDisabledTextColor(Color.white);
		script.setBackground(Color.white);
		script.setText("");
		script.setEditable(false);
		example.setText("AaBbCc");
		example.setEditable(false);
		example.setBorder(new EmptyBorder(0,0,0,0));
		// setting the components content
        styles=new Hashtable();
        styles.put("Plain",new Integer(Font.PLAIN));
        styles.put("Bold",new Integer(Font.BOLD));
        styles.put("Italic",new Integer(Font.ITALIC));
        styles.put("Bold Italic",new Integer(Font.BOLD | Font.ITALIC));
        String[] styleNames = {"Plain","Bold","Italic","Bold Italic"}; 
        ranges=new Hashtable();
        ranges.put("Occidental",new Integer(ASCII_BASE));
        ranges.put("Greek",new Integer(GREEK_BASE));
        ranges.put("Hebrew",new Integer(HEBREW_BASE));
        ranges.put("Arabic",new Integer(ARABIC_BASE));
        ranges.put("Thai",new Integer(THAI_BASE));
        ranges.put("Georgian",new Integer(GEORGIAN_BASE));
        ranges.put("Hiragana",new Integer(HIRAGANA_BASE));
        ranges.put("Katakana",new Integer(KATAKANA_BASE));
        ranges.put("Hangul",new Integer(HANGUL_BASE));
        ranges.put("Kanji",new Integer(KANJI_BASE));
        String[] rangeNames = {"Occidental","Greek","Hebrew","Arabic","Thai","Georgian","Hiragana","Katakana","Hangul","Kanji"};

        GraphicsEnvironment ge=GraphicsEnvironment.getLocalGraphicsEnvironment();
        String[] fontNames=ge.getAvailableFontFamilyNames();
        int i;
        
        for (i=0;i<fontNames.length;i++) {
			if (!fontNames[i].equals("")) {
				fontList.add(fontNames[i]);
			}
		}
		fontList.addItemListener(new FontListener());
		
		for (i=0;i<styleNames.length;i++) {
			styleList.add(styleNames[i]);
		}
		styleList.addItemListener(new StyleListener());

		sizeList.add(String.valueOf(8));
		sizeList.add(String.valueOf(9));
		sizeList.add(String.valueOf(10));
		sizeList.add(String.valueOf(11));
		sizeList.add(String.valueOf(12));
		sizeList.add(String.valueOf(14));
		sizeList.add(String.valueOf(16));
		sizeList.add(String.valueOf(18));
		sizeList.add(String.valueOf(20));
		sizeList.add(String.valueOf(22));
		sizeList.add(String.valueOf(24));
		sizeList.add(String.valueOf(26));
		sizeList.add(String.valueOf(28));
		sizeList.add(String.valueOf(36));
		sizeList.add(String.valueOf(48));
		sizeList.add(String.valueOf(72));
		sizeList.addItemListener(new SizeListener());
		
		for (i=0;i<rangeNames.length;i++) {
			scriptList.add(rangeNames[i]);
		}
		scriptList.addItemListener(new RangeListener());
		scriptList.select(0);
	}


	int toInt(String s) {
		int j=0;
		for (int i=0;i<s.length();i++)
			j=j*10+s.charAt(i)-'0';
		return j;
	}


	public void refresh() {
		font.setText(fontName);
		if (fontStyle==Font.PLAIN) style.setText("Plain");
		if (fontStyle==Font.BOLD) style.setText("Bold");
		if (fontStyle==Font.ITALIC) style.setText("Italic");
		if (fontStyle==(Font.BOLD | Font.ITALIC)) style.setText("Bold Italic");
		size.setText(String.valueOf(fontSize));
		script.setText((String)scriptList.getSelectedItem());
		res=new Font(fontName,fontStyle,(int)(fontSize/0.72));
		example.setFont(res);
		if (fontScript==ASCII_BASE)	example.setText("AaBbCc");
		if (fontScript==GREEK_BASE)	example.setText("\u0391\u03b1\u0392\u03b2\u0393\u03b3");
		if (fontScript==HEBREW_BASE)	example.setText("\u05d0\u05d1\u05d2\u05d3\u05d4\u05d5");
		if (fontScript==ARABIC_BASE)	example.setText("\u0621\u0622\u0623\u0624\u0625\u0626");
		if (fontScript==THAI_BASE)	example.setText("\u0e01\u0e02\u0e03\u0e04\u0e05\u0e06");
		if (fontScript==GEORGIAN_BASE)	example.setText("\u10a0\u10a1\u10a2\u10a3\u10a4\u10a5");
		if (fontScript==HIRAGANA_BASE)	example.setText("\u3041\u3042\u3043\u3044\u3045\u3046");
		if (fontScript==KATAKANA_BASE)	example.setText("\u30a1\u30a2\u30a3\u30a4\u30a5\u30a6");
		if (fontScript==HANGUL_BASE)	example.setText("\u3131\u3132\u3133\u3134\u3135\u3136");
		if (fontScript==KANJI_BASE)	example.setText("\u4e07\u4e08\u4e09\u4e0a\u4e0b\u4e5f");
	}


	class FontListener implements ItemListener {
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange()!=ItemEvent.SELECTED) return;
			font.setText((String)fontList.getSelectedItem());
			fontName=font.getText();
			refresh();
		}
	}


	class StyleListener implements ItemListener {
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange()!=ItemEvent.SELECTED) return;
			style.setText((String)styleList.getSelectedItem());
			fontStyle=((Integer)styles.get(styleList.getSelectedItem())).intValue();
			refresh();
		}
	}


	class SizeListener implements ItemListener {
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange()!=ItemEvent.SELECTED) return;
			size.setText((String)sizeList.getSelectedItem());
			fontSize=toInt(size.getText());
			refresh();
		}
	}


	class RangeListener implements ItemListener {
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange()!=ItemEvent.SELECTED) return;
			script.setText((String)scriptList.getSelectedItem());
			fontScript=((Integer)ranges.get(scriptList.getSelectedItem())).intValue();
			refresh();
		}
	}


	class OKListener implements MouseListener {
		public void mousePressed(MouseEvent e) {}
		public void mouseReleased(MouseEvent e) {
			Preferences.temp.textFont=new Font(fontName,fontStyle,(int)(fontSize/0.72));
			Preferences.temp.textFontSize=fontSize;
			GlobalPreferenceFrame.getFrame().textFont.setText(""+Preferences.temp.textFont.getFontName()+"  "+Preferences.temp.textFontSize+"");
			setVisible(false);
			e.consume();
		}
		public void mouseMoved(MouseEvent e) {}
		public void mouseDragged(MouseEvent e) {}
		public void mouseEntered(MouseEvent e) {}
		public void mouseExited(MouseEvent e) {}
		public void mouseClicked(MouseEvent e) {}
	}


	class CANCELListener implements MouseListener {
		public void mousePressed(MouseEvent e) {}
		public void mouseReleased(MouseEvent e) {
			setVisible(false);
			e.consume();
		}
		public void mouseMoved(MouseEvent e) {}
		public void mouseDragged(MouseEvent e) {}
		public void mouseEntered(MouseEvent e) {}
		public void mouseExited(MouseEvent e) {}
		public void mouseClicked(MouseEvent e) {}
	}


class FontList extends List {
    public FontList(int rows) {
        super(rows);
    }
    public Dimension getPreferredSize() {
        return new Dimension(150,100);
    }
}


class StyleList extends List {
    public StyleList(int rows) {
        super(rows);
    }
    public Dimension getPreferredSize() {
        return new Dimension(120,100);
    }
}


class SizeList extends List {
    public SizeList(int rows) {
        super(rows);
    }
    public Dimension getPreferredSize() {
        return new Dimension(50,100);
    }
}


class ScriptList extends List {
    public ScriptList(int rows) {
        super(rows);
    }
    public Dimension getPreferredSize() {
        return new Dimension(50,50);
    }
}


}
