 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.io.*;
import java.lang.System;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.nio.charset.Charset;
import java.nio.channels.*;

/*
 * This class is used to display the text
 *
 */
public class TextFrame extends JInternalFrame implements InternalFrameListener {
	MyTextArea text=new MyTextArea();
	JScrollPane scroll;
	public static TextFrame frame;
	public static boolean FILE_TOO_LARGE=false;
	JLabel ligne1=new JLabel("");
	JLabel ligne2=new JLabel("");
	
		
	public TextFrame() {
		super("",true,true,true,true);
		JPanel top=new JPanel();
		top.setOpaque(true);
		top.setLayout(new BorderLayout());
		top.setBorder(new EmptyBorder(2,2,2,2));
		scroll=new JScrollPane(text);
		scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		JPanel middle=new JPanel();
		middle.setOpaque(true);
		middle.setLayout(new BorderLayout());
		middle.setBorder(BorderFactory.createLoweredBevelBorder());
		middle.add(scroll);
		JPanel up=new JPanel();
		up.setOpaque(true);
		up.setBorder(new EmptyBorder(2,2,2,2));
		up.setLayout(new GridLayout(2,1));
		up.add(ligne1);
		up.add(ligne2);
		top.add(up,BorderLayout.NORTH);
		top.add(middle,BorderLayout.CENTER);
		setContentPane(top);
		pack();
		setBounds(100,100,800,600);
		setVisible(false);
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addInternalFrameListener(this);
	}
	
	
	public static void init() {
		frame=new TextFrame();
	}

	
	public static TextFrame getFrame() {
		return frame;
	}

	
	public static void loadStatistics() {
		frame.ligne1.setText("");
		frame.ligne2.setText("");
		FileInputStream source;
		String s;
		s=readLine(Config.getCurrentSntDir()+"stats.n");
		if (s!=null) frame.ligne1.setText(" "+s);
		s=readLine(Config.getCurrentSntDir()+"dlf.n");
		if (s==null) return;
		int dlf_entries=new Integer(s).intValue();
		s=readLine(Config.getCurrentSntDir()+"dlc.n");
		if (s==null) return;
		int dlc_entries=new Integer(s).intValue();
		s=readLine(Config.getCurrentSntDir()+"err.n");
		if (s==null) return;
		int err_entries=new Integer(s).intValue();
		File f=new File(Config.getCurrentSntDir()+"stat_dic.n");
		if (!f.exists()) {return;}
		if (!f.canRead()) {return;}
		int simple_total;
		int compound_total;
		int err_total;
		try {
			source=new FileInputStream(f);
			char c=read_char(source);
			if (c!=0xFEFF) {
				return;
			}
			simple_total=new Integer(readLine(source)).intValue();
			compound_total=new Integer(readLine(source)).intValue();
			err_total=new Integer(readLine(source)).intValue();
			source.close();
		} catch (FileNotFoundException e) {return;}
		  catch (IOException e) {return;}
		s=" "+String.valueOf(simple_total)+" (";
		s=s+String.valueOf(dlf_entries)+" diff) simple word";
		if (dlf_entries>1) s=s+"s";
		s=s+", ";
		s=s+String.valueOf(compound_total)+" (";
		s=s+String.valueOf(dlc_entries)+") compound word";
		if (dlc_entries>1) s=s+"s";
		s=s+", ";
		s=s+String.valueOf(err_total)+" (";
		s=s+String.valueOf(err_entries)+") unknown token";
		if (err_entries>1) s=s+"s";
		frame.ligne2.setText(s);
	}
	
		
	public static void loadText(String snt) {
		UnitexFrame.mainFrame.preprocessText.setEnabled(true);
		UnitexFrame.mainFrame.applyLexicalResources.setEnabled(true);
		UnitexFrame.mainFrame.locatePattern.setEnabled(true);
		UnitexFrame.mainFrame.displayLocatedSequences.setEnabled(true);
		UnitexFrame.mainFrame.constructFst.setEnabled(true);
		UnitexFrame.mainFrame.close.setEnabled(true);
		loadStatistics();
		frame.text.setFont(Config.getCurrentTextFont());
		frame.text.setLineWrap(true);
		frame.text.setEditable(false);
		File FILE=new File(snt);
		if (FILE.length()<=2) {
			FILE_TOO_LARGE=true;
			frame.text.setDocument(new PlainDocument());
			frame.text.setText(Config.EMPTY_FILE_MESSAGE);
		}
		else if (FILE.length()<Config.MAX_FILE_SIZE) {
			try {
				frame.text.load(FILE);
			} catch(IOException E) {
				frame.text.setDocument(new PlainDocument());
				frame.text.setText(Config.ERROR_WHILE_READING_FILE_MESSAGE);
				FILE_TOO_LARGE=true;
			}
			FILE_TOO_LARGE=false;
		}
		else {
			FILE_TOO_LARGE=true;
			frame.text.setDocument(new PlainDocument());
			frame.text.setText(Config.FILE_TOO_LARGE_MESSAGE);
		}
		frame.setTitle(snt);
		frame.setVisible(true);
		try { 
		    frame.setSelected(true); 
		    frame.setIcon(false);
		} catch (java.beans.PropertyVetoException e2) {}
	}
	
	
	public static void closeText() {
		frame.setVisible(false);
		try { 
		    frame.setIcon(false);
		} catch (java.beans.PropertyVetoException e2) {}
		frame.text.setDocument(new PlainDocument());
		System.gc();
	}
	
	public void internalFrameActivated(InternalFrameEvent e) {}
	public void internalFrameClosed(InternalFrameEvent e) {}
	public void internalFrameClosing(InternalFrameEvent e) {
		try {
			setIcon(true);
		} catch (java.beans.PropertyVetoException e2) {}
	}
	public void internalFrameDeactivated(InternalFrameEvent e) {}
	public void internalFrameDeiconified(InternalFrameEvent e) {}
	public void internalFrameIconified(InternalFrameEvent e) {}
	public void internalFrameOpened(InternalFrameEvent e) {}



	public static int toInt(byte b) {
		if (b>0) return b;
		else return 256+b;
	}


	public static char read_char(FileInputStream f) {
		byte[] b=new byte[2];
		char c;
		try {
			f.read(b);
		} catch(IOException e) {e.printStackTrace();}
		c=(char)(toInt(b[1])*256+toInt(b[0]));
		if (c==0x0d) {
			read_char(f);
			c='\n';
		}
		return c;
	}


	public static String readLine(FileInputStream f) {
		char c;
		String s="";
		while ((c=read_char(f))!='\n') {
			s=s+c;
		}
		return s;
	}


	public static String readLine(String name) {
		File f=new File(name);
		if (!f.exists()) {return null;}
		if (!f.canRead()) {return null;}
		String res;
		FileInputStream source;
		try {
			source=new FileInputStream(f);
			char c=read_char(source);
			if (c!=0xFEFF) {
				return null;
			}
			res=readLine(source);
			source.close();
		} catch (FileNotFoundException e) {return null;}
		  catch (IOException e) {return null;}
		  return res;
	}


}