 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.nio.charset.Charset;
import java.nio.channels.*;

/*
 * This class is used to display the text token list
 *
 */
public class TokensFrame extends JInternalFrame implements InternalFrameListener {
	MyTextArea text=new MyTextArea();
	JScrollPane scroll;
	public static TokensFrame frame;
	public static boolean FILE_TOO_LARGE=false;
	JPanel buttonsPanel=new JPanel();
	JButton byFrequence=new JButton("By Frequence");
	JButton byCharOrder=new JButton("By Char Order");
	

	public TokensFrame() {
		super("Token list",true,true,true,true);
		JPanel top=new JPanel();
		top.setOpaque(true);
		top.setLayout(new BorderLayout());
		scroll=new JScrollPane(text);
		scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		constructButtonsPanel();
		top.add(buttonsPanel,BorderLayout.NORTH);
		top.add(scroll,BorderLayout.CENTER);
		setContentPane(top);
		pack();
		setBounds(50,200,300,450);
		setVisible(false);
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addInternalFrameListener(this);
	}
	
	
	public void constructButtonsPanel() {
		buttonsPanel.setOpaque(true);
		buttonsPanel.setLayout(new GridLayout(1,2));
		byFrequence.addMouseListener(new ByFrequenceButtonListener());
		byCharOrder.addMouseListener(new ByCharOrderButtonListener());
		JPanel tmp1=new JPanel();
		tmp1.setOpaque(true);
		tmp1.setLayout(new BorderLayout());
		tmp1.setBorder(new EmptyBorder(5,5,5,5));
		tmp1.add(byFrequence,BorderLayout.CENTER);
		JPanel tmp2=new JPanel();
		tmp2.setOpaque(true);
		tmp2.setLayout(new BorderLayout());
		tmp2.setBorder(new EmptyBorder(5,5,5,5));
		tmp2.add(byCharOrder,BorderLayout.CENTER);
		buttonsPanel.add(tmp1);
		buttonsPanel.add(tmp2);
	}
	
	
	
	public static void init() {
		frame=new TokensFrame();
	}

	
	public static TokensFrame getFrame() {
		return frame;
	}

	
	
	public static void loadTokens(String tokens) {
		frame.text.setFont(Config.getCurrentTextFont());
		frame.text.setLineWrap(true);
		frame.text.setEditable(false);
		File FILE=new File(tokens);
		if (FILE.length()<=2) {
			FILE_TOO_LARGE=true;
			frame.text.setDocument(new PlainDocument());
			frame.text.setText(Config.EMPTY_FILE_MESSAGE);
		}
		else if (FILE.length()<Config.MAX_FILE_SIZE) {	
			try {
				frame.text.load(FILE);
			} catch (java.io.IOException e) {
				FILE_TOO_LARGE=true; 
				frame.text.setDocument(new PlainDocument());
				frame.text.setText(Config.ERROR_WHILE_READING_FILE_MESSAGE);
				return;
				}
			FILE_TOO_LARGE=false;
		}
		else {
			FILE_TOO_LARGE=true;
			frame.text.setDocument(new PlainDocument());
			frame.text.setText(Config.FILE_TOO_LARGE_MESSAGE);
		}
		frame.setVisible(true);
		try { 
		    frame.setIcon(true);
		    frame.setSelected(true); 
		} catch (java.beans.PropertyVetoException e2) {}
	}
	
	
	public static void closeTokens() {
		frame.setVisible(false);
		frame.text.setDocument(new PlainDocument());
		try { 
		    frame.setIcon(false);
		} catch (java.beans.PropertyVetoException e2) {}
		System.gc();
	}
	
	
	public void internalFrameActivated(InternalFrameEvent e) {}
	public void internalFrameClosed(InternalFrameEvent e) {}
	public void internalFrameClosing(InternalFrameEvent e) {
		try {
			setIcon(true);
		} catch (java.beans.PropertyVetoException e2) {}
	}
	public void internalFrameDeactivated(InternalFrameEvent e) {}
	public void internalFrameDeiconified(InternalFrameEvent e) {}
	public void internalFrameIconified(InternalFrameEvent e) {}
	public void internalFrameOpened(InternalFrameEvent e) {}



class ByFrequenceButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		loadTokens(Config.getCurrentSntDir()+"tok_by_freq.txt");
		try { 
		    frame.setIcon(false);
		    frame.setSelected(true); 
		} catch (java.beans.PropertyVetoException e2) {}
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}



class ByCharOrderButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		loadTokens(Config.getCurrentSntDir()+"tok_by_alph.txt");
		try { 
		    frame.setIcon(false);
		    frame.setSelected(true); 
		} catch (java.beans.PropertyVetoException e2) {}
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}



}