 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.metal.*;
import java.awt.datatransfer.*;
import java.awt.print.*;
import javax.swing.filechooser.*;
import javax.swing.text.*;

/*
 * This is the main frame of the Unitex platform
 */
public class UnitexFrame extends JFrame {

	JMenuBar menuBar;
	public static JDesktopPane desktop;
	public static final Integer DOCLAYER = new Integer(5);
	public static final Integer TOOLLAYER = new Integer(6);
	public static final Integer HELPLAYER = new Integer(7);
	public static Clipboard clip=new Clipboard("Unitex clipboard");
	static UnitexFrame mainFrame;    
	static Dimension screenSize;
	public static Vector frames=new Vector();	// used to handle all the graph frames
	public static GraphFrame currentFrame=null;	// the current focused frame
	PrinterJob printerJob;
	PageFormat pageFormat;
	
	
	public UnitexFrame() {
        	super(Version.version);
        	Config.initConfig();
        	final int inset = 50;
        	screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		setBounds ( inset, inset, screenSize.width - inset*2, screenSize.height - inset*2 );
		buildContent();
		buildMenus();
		mainFrame=this;
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				quit();
			}
		});
		Preferences.setDefaultPreferences();
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		setTitle(Version.version+" - current language is "+Config.getCurrentLanguage());
		Console.init();
		TextFrame.init();
		ApplyLexicalResourcesFrame.init();
		LocateFrame.init();
      		ConcordanceParameterFrame.init();
		DelaFrame.init();
		InflectFrame.init();
		TokensFrame.init();
		TextDicFrame.init();
		CheckDicFrame.init();
		CheckResultFrame.init();
		GlobalPreferenceFrame.init();
		LexiconGrammarFrame.init();
		desktop.add(Console.getFrame(),DOCLAYER);
		desktop.add(TextFrame.getFrame(),DOCLAYER);
		desktop.add(ApplyLexicalResourcesFrame.getFrame(),DOCLAYER);
		desktop.add(LocateFrame.getFrame(),DOCLAYER);
		desktop.add(ConcordanceParameterFrame.getFrame(),DOCLAYER);
		desktop.add(DelaFrame.getFrame(),DOCLAYER);
		desktop.add(InflectFrame.getFrame(),DOCLAYER);
		desktop.add(TokensFrame.getFrame(),DOCLAYER);
		desktop.add(TextDicFrame.getFrame(),DOCLAYER);
		desktop.add(CheckDicFrame.getFrame(),DOCLAYER);
		desktop.add(CheckResultFrame.getFrame(),DOCLAYER);
		desktop.add(GlobalPreferenceFrame.getFrame(),DOCLAYER);
		desktop.add(LexiconGrammarFrame.getFrame(),DOCLAYER);
    }



    public void buildMenus() {
        menuBar = new JMenuBar();
		menuBar.setOpaque(true);
		JMenu text = buildTextMenu();
		JMenu DELA = buildDELAMenu();
		JMenu fsGraph = buildFsGraphMenu();
		JMenu lexiconGrammar = buildLexiconGrammarMenu();
		JMenu edit = buildEditMenu();
		JMenu windows = buildWindowsMenu();
		JMenu info = buildInfoMenu();
		text.setMnemonic(KeyEvent.VK_T);
		DELA.setMnemonic(KeyEvent.VK_D);
		fsGraph.setMnemonic(KeyEvent.VK_G);
		lexiconGrammar.setMnemonic(KeyEvent.VK_L);
		edit.setMnemonic(KeyEvent.VK_E);
		windows.setMnemonic(KeyEvent.VK_W);
		info.setMnemonic(KeyEvent.VK_I);
		menuBar.add(text);
		menuBar.add(DELA);
		menuBar.add(fsGraph);
		menuBar.add(lexiconGrammar);
		menuBar.add(edit);
		menuBar.add(windows);
		menuBar.add(info);
		setJMenuBar(menuBar);	
    }


	JMenuItem open;
	JMenuItem preprocessText;
	JMenuItem applyLexicalResources;
	JMenuItem locatePattern;
	JMenuItem displayLocatedSequences;
	JMenuItem constructFst;
	JMenuItem close;
	
    public JMenu buildTextMenu() {
		JMenu text = new JMenu("Text");
		open = new JMenuItem("Open...");
		open.setEnabled(true);
		open.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
	                	SwingUtilities.invokeLater(new Runnable() {
	                		public void run() {
						openText();
					}
				});
		}});
		preprocessText = new JMenuItem("Preprocess Text...");
	        preprocessText.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						String txt=Config.getCurrentSnt();
						txt=txt.substring(0,txt.length()-3);
						txt=txt+"txt";
						new PreprocessFrame(txt,Config.getCurrentSnt());
					}
				});
		}});
		preprocessText.setEnabled(false);
		JMenuItem changeLang = new JMenuItem("Change Language...");
		changeLang.setEnabled(true);
		changeLang.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
	                	SwingUtilities.invokeLater(new Runnable() {
	                		public void run() {
				    		Config.changeLanguage();
				    	}
				});
		}});
		applyLexicalResources = new JMenuItem("Apply Lexical Resources...");
		applyLexicalResources.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
	                	SwingUtilities.invokeLater(new Runnable() {
	                		public void run() {
				    		ApplyLexicalResourcesFrame.showFrame();
				    	}
				});
		}});
		applyLexicalResources.setEnabled(false);
		/*JMenuItem disambiguation = new JMenuItem("Disambiguation...");
		disambiguation.setEnabled(false);
		*/locatePattern = new JMenuItem("Locate Pattern...");
		locatePattern.setAccelerator(KeyStroke.getKeyStroke('L',Event.CTRL_MASK ));
		locatePattern.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
	                	SwingUtilities.invokeLater(new Runnable() {
	                		public void run() {
		   				LocateFrame.showFrame();
		   			}
				});
		}});
		locatePattern.setEnabled(false);
		displayLocatedSequences = new JMenuItem("Display Located Sequences...");
		displayLocatedSequences.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
	                	SwingUtilities.invokeLater(new Runnable() {
	                		public void run() {
						ConcordanceParameterFrame.showFrame();
				   	}
				});
		}});
		displayLocatedSequences.setEnabled(false);
		constructFst = new JMenuItem("Construct FST-Text...");
		constructFst.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
	                	SwingUtilities.invokeLater(new Runnable() {
	                		public void run() {
		   				new ConstructFstFrame();
		   			}
				});
		}});
		constructFst.setEnabled(false);
		close = new JMenuItem("Close Text...");
		close.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				closeText();
		}});
		close.setEnabled(false);
		JMenuItem quit = new JMenuItem("Quit Unitex");
		quit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
	               		quit();
		}});
		text.add(open);
		//text.add(textEditor);
		text.add(preprocessText);
		text.addSeparator();
		text.add(changeLang);
		text.addSeparator();
		text.add(applyLexicalResources);
		//text.add(disambiguation);
		text.addSeparator();
		text.add(locatePattern);
		text.add(displayLocatedSequences);
		text.addSeparator();
		text.add(constructFst);
		text.addSeparator();
		text.add(close);
		text.add(quit);
		return text;
    }


	JMenuItem checkDelaFormat;
	JMenuItem sortDictionary;
	JMenuItem inflect;
	JMenuItem compressIntoFST;
	JMenuItem closeDela;
	
	public JMenu buildDELAMenu() {
		JMenu DELA = new JMenu("DELA");
		JMenuItem open = new JMenuItem("Open...");
		open.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
	                		public void run() {
				   		openDELA();
				   	}
				});
		}});
		open.setEnabled(true);
		checkDelaFormat = new JMenuItem("Check Format...");
		checkDelaFormat.setAccelerator(KeyStroke.getKeyStroke('K',Event.CTRL_MASK ));
		checkDelaFormat.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
	                		public void run() {
				   		CheckDicFrame.showFrame();
		   			}
		   		});
		}});
		checkDelaFormat.setEnabled(false);
		sortDictionary = new JMenuItem("Sort Dictionary");
		sortDictionary.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
	                	SwingUtilities.invokeLater(new Runnable() {
	                		public void run() {
		   				sortDELA();
		   			}
				});
		}});
		sortDictionary.setEnabled(false);
		inflect = new JMenuItem("Inflect...");
		inflect.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
	                		public void run() {
		   				InflectFrame.showFrame();
		   			}
				});
		}});
		inflect.setEnabled(false);
		compressIntoFST = new JMenuItem("Compress into FST");
		compressIntoFST.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
	                	SwingUtilities.invokeLater(new Runnable() {
	                		public void run() {
		   				compressDELA();
		   			}
				});
		}});
		compressIntoFST.setEnabled(false);
		closeDela = new JMenuItem("Close");
		closeDela.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
	                	SwingUtilities.invokeLater(new Runnable() {
	                		public void run() {
		   				closeDELA();
		   			}
				});
		}});
		closeDela.setEnabled(false);
		DELA.add(open);
		DELA.addSeparator();
		DELA.add(checkDelaFormat);
		DELA.add(sortDictionary);
		DELA.add(inflect);
		DELA.add(compressIntoFST);
		DELA.addSeparator();
		DELA.add(closeDela);
		return DELA;
    }


	public JMenu buildFsGraphMenu() {
		JMenu fsGraph = new JMenu("FSGraph");
		JMenuItem New = new JMenuItem("New");
		New.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
	                	SwingUtilities.invokeLater(new Runnable() {
	                		public void run() {
						newDocument();
				   	}
				});
		}});
		JMenuItem open = new JMenuItem("Open...");
		open.setAccelerator(KeyStroke.getKeyStroke('O',Event.CTRL_MASK ));
		open.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						openGraph();
					}
				});
		}});
		JMenuItem find = new JMenuItem("Find...");
		find.setEnabled(false);
		JMenuItem save = new JMenuItem("Save");
		save.setAccelerator(KeyStroke.getKeyStroke('S',Event.CTRL_MASK ));
		save.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						saveGraph(currentFrame);
					}
				});
		}});
		JMenuItem saveAs = new JMenuItem("Save as...");
		saveAs.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						saveAsGraph(currentFrame);
					}
				});
		}});
		JMenuItem saveAll = new JMenuItem("Save All");
		saveAll.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						saveAllGraphs();
					}
				});
		}});
		JMenuItem setup = new JMenuItem("Page Setup");
		setup.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						pageSetup();
					}
				});
		}});
		JMenuItem print = new JMenuItem("Print...");
		print.setAccelerator(KeyStroke.getKeyStroke('P',Event.CTRL_MASK ));
		print.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						if (currentFrame!=null) {
							printFrame(currentFrame);
						} else {
							if (TextAutomatonFrame.getFrame()!=null && 
							    TextAutomatonFrame.getFrame().isSelected()) {
								printTextAutomatonFrame(TextAutomatonFrame.getFrame());
							}
						}
					}
				});
		}});
		JMenuItem printAll = new JMenuItem("Print All...");
		printAll.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						printAllFrames();
					}
				});
		}});
		JMenu tools = new JMenu("Tools");
			JMenuItem sortNodeLabel = new JMenuItem("Sort Node Label");
			sortNodeLabel.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							if (currentFrame!=null) {
								currentFrame.sortNodeLabel();
							}
						}
					});
			}});
			JMenuItem compileFST = new JMenuItem("Compile FST2");
			compileFST.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							compileGraph();
						}
					});
			}});
			tools.add(sortNodeLabel);
			tools.addSeparator();
			tools.add(compileFST);
		JMenu format = new JMenu("Format");
			JMenuItem alignment = new JMenuItem("Alignment...");
			alignment.setAccelerator(KeyStroke.getKeyStroke('M',Event.CTRL_MASK ));
			alignment.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							if (currentFrame!=null) {
								new GraphPreferenceMenu(currentFrame.graphicalZone.isGrid,currentFrame.graphicalZone.nPixels);
							}
						}
					});
			}});
			JMenuItem antialiasing = new JMenuItem("Antialiasing...");
			antialiasing.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							if (currentFrame!=null) {
								currentFrame.changeAntialiasingValue();
							} else {
								if (TextAutomatonFrame.getFrame()!=null && 
								    TextAutomatonFrame.getFrame().isSelected()) {
									TextAutomatonFrame.getFrame().changeAntialiasingValue();
								}
							}
						}
					});
			}});
			JMenuItem presentation = new JMenuItem("Presentation...");
			presentation.setAccelerator(KeyStroke.getKeyStroke('R',Event.CTRL_MASK ));
			presentation.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							if (currentFrame!=null) {
								new GraphPresentationMenu();
							}
						}
					});
			}});
			JMenuItem graphSize = new JMenuItem("Graph Size...");
			graphSize.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							if (currentFrame!=null) {
								new GraphSizeMenu();
							}
						}
					});
			}});
			format.add(antialiasing);
			format.addSeparator();
			format.add(alignment);
			format.add(presentation);
			format.add(graphSize);
		JMenu zoom = new JMenu("Zoom");
		ButtonGroup groupe=new ButtonGroup();
		JRadioButtonMenuItem fitInScreen=new JRadioButtonMenuItem("Fit in screen");
		JRadioButtonMenuItem fitInWindow=new JRadioButtonMenuItem("Fit in window");
		JRadioButtonMenuItem fit60=new JRadioButtonMenuItem("60%");
		JRadioButtonMenuItem fit80=new JRadioButtonMenuItem("80%");
		JRadioButtonMenuItem fit100=new JRadioButtonMenuItem("100%");
		JRadioButtonMenuItem fit120=new JRadioButtonMenuItem("120%");
		JRadioButtonMenuItem fit140=new JRadioButtonMenuItem("140%");
		groupe.add(fitInScreen);
		groupe.add(fitInWindow);
		groupe.add(fit60);
		groupe.add(fit80);
		groupe.add(fit100);
		fit100.setSelected(true);
		groupe.add(fit120);
		groupe.add(fit140);
		fitInScreen.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						if (currentFrame!=null) {
							currentFrame.removeComponentListener(currentFrame.compListener);
							double scale_x=(double)screenSize.width/(double)currentFrame.graphicalZone.Width;
							double scale_y=(double)screenSize.height/(double)currentFrame.graphicalZone.Height;
							if (scale_x<scale_y) 
								currentFrame.setScaleFactor(scale_x);
							else currentFrame.setScaleFactor(scale_y);
						}
					}
				});
		}});
		fitInWindow.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						if (currentFrame!=null) {
							Dimension d=currentFrame.scroll.getSize();
							double scale_x=(double)(d.width-3)/(double)currentFrame.graphicalZone.Width;
							double scale_y=(double)(d.height-3)/(double)currentFrame.graphicalZone.Height;
							if (scale_x<scale_y) 
								currentFrame.setScaleFactor(scale_x);
							else currentFrame.setScaleFactor(scale_y);
							currentFrame.compListener=new ComponentListener() {
								GraphFrame current=currentFrame;
								public void componentHidden(ComponentEvent e) {}
								public void componentMoved(ComponentEvent e) {}
								public void componentResized(ComponentEvent e) {
									Dimension d=current.scroll.getSize();
									double scale_x=(double)(d.width-3)/(double)current.graphicalZone.Width;
									double scale_y=(double)(d.height-3)/(double)current.graphicalZone.Height;
									if (scale_x<scale_y)
										current.setScaleFactor(scale_x);
									else current.setScaleFactor(scale_y);
								}
								public void componentShown(ComponentEvent e) {}
							};
							currentFrame.addComponentListener(currentFrame.compListener);
						}
					}
				});
		}});
		fit60.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						if (currentFrame!=null) {
							currentFrame.removeComponentListener(currentFrame.compListener);
							currentFrame.setScaleFactor(0.6);
						}
					}
				});
		}});
		fit80.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						if (currentFrame!=null) {
							currentFrame.removeComponentListener(currentFrame.compListener);
							currentFrame.setScaleFactor(0.8);
						}
					}
				});
		}});
		fit100.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						if (currentFrame!=null) {
							currentFrame.removeComponentListener(currentFrame.compListener);
							currentFrame.setScaleFactor(1.0);
						}
					}
				});
		}});
		fit120.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						if (currentFrame!=null) {
							currentFrame.removeComponentListener(currentFrame.compListener);
							currentFrame.setScaleFactor(1.2);
						}
					}
				});
		}});
		fit140.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						if (currentFrame!=null) {
							currentFrame.removeComponentListener(currentFrame.compListener);
							currentFrame.setScaleFactor(1.4);
						}
					}
				});
		}});
		zoom.add(fitInScreen);
		zoom.add(fitInWindow);
		zoom.add(fit60);
		zoom.add(fit80);
		zoom.add(fit100);
		zoom.add(fit120);
		zoom.add(fit140);
		JMenuItem close = new JMenuItem("Close all");
		close.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						closeAll();
					}
				});
		}});
		fsGraph.add(New);
		fsGraph.add(open);
		//fsGraph.add(find);
		fsGraph.addSeparator();
		fsGraph.add(save);
		fsGraph.add(saveAs);
		fsGraph.add(saveAll);
		fsGraph.addSeparator();
		fsGraph.add(setup);
		fsGraph.add(print);
		fsGraph.add(printAll);
		fsGraph.addSeparator();
		fsGraph.add(tools);
		fsGraph.add(format);
		fsGraph.add(zoom);
		fsGraph.addSeparator();
		fsGraph.add(close);
		return fsGraph;
	}


	
	JMenuItem openLexiconGrammar;
	JMenuItem compileLexiconGrammar;
	JMenuItem closeLexiconGrammar;
	
	public JMenu buildLexiconGrammarMenu() {
		JMenu lexiconGrammar = new JMenu("Lexicon-Grammar");
		openLexiconGrammar = new JMenuItem("Open...");
		openLexiconGrammar.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				final ActionEvent E=e;
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						openLexiconGrammarTable();
					}
				});
		}});
		openLexiconGrammar.setEnabled(true);
		compileLexiconGrammar = new JMenuItem("Compile to GRF...");
		compileLexiconGrammar.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				final ActionEvent E=e;
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						LexiconGrammarFrame.showFrame();
					}
				});
		}});
		compileLexiconGrammar.setEnabled(false);
		closeLexiconGrammar = new JMenuItem("Close");
		closeLexiconGrammar.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				final ActionEvent E=e;
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						compileLexiconGrammar.setEnabled(false);
						LexiconGrammarTableFrame.closeFrame();
					}
				});
		}});
		closeLexiconGrammar.setEnabled(false);
		lexiconGrammar.add(openLexiconGrammar);
		lexiconGrammar.add(compileLexiconGrammar);
		lexiconGrammar.add(closeLexiconGrammar);
		return lexiconGrammar;
	}


	public JMenu buildEditMenu() {
		JMenu edit = new JMenu("Edit");
		JMenuItem cut = new JMenuItem("Cut");
		cut.setAccelerator(KeyStroke.getKeyStroke('X',Event.CTRL_MASK ));
		cut.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				final ActionEvent E=e;
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						if (TextAutomatonFrame.getFrame()!=null && 
							    TextAutomatonFrame.getFrame().isSelected()) {
								TextAutomatonFrame.getFrame().graphicalZone.texte.cut.actionPerformed(E);
								TextAutomatonFrame.getFrame().graphicalZone.repaint();
							}
						else if (currentFrame!=null) {
							currentFrame.graphicalZone.texte.cut.actionPerformed(E);
							currentFrame.graphicalZone.repaint();
						} 
					}
				});
		}});
		cut.setEnabled(true);
		JMenuItem copy = new JMenuItem("Copy");
		copy.setAccelerator(KeyStroke.getKeyStroke('C',Event.CTRL_MASK ));
		copy.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				final ActionEvent E=e;
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						if (TextAutomatonFrame.getFrame()!=null && 
							    TextAutomatonFrame.getFrame().isSelected()) {
								TextAutomatonFrame.getFrame().graphicalZone.texte.specialCopy.actionPerformed(E);
								TextAutomatonFrame.getFrame().graphicalZone.repaint();
							}
						else if (currentFrame!=null) {
							currentFrame.graphicalZone.texte.specialCopy.actionPerformed(E);
							currentFrame.graphicalZone.repaint();
						} 
					}
				});
		}});
		copy.setEnabled(true);
		JMenuItem paste = new JMenuItem("Paste");
		paste.setAccelerator(KeyStroke.getKeyStroke('V',Event.CTRL_MASK ));
		paste.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				final ActionEvent E=e;
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						if (TextAutomatonFrame.getFrame()!=null && 
							    TextAutomatonFrame.getFrame().isSelected()) {
								TextAutomatonFrame.getFrame().graphicalZone.texte.specialPaste.actionPerformed(E);
								TextAutomatonFrame.getFrame().graphicalZone.repaint();
							}
						else if (currentFrame!=null) {
							currentFrame.graphicalZone.texte.specialPaste.actionPerformed(E);
							currentFrame.graphicalZone.repaint();
						} 
					}
				});
		}});
		paste.setEnabled(true);
		JMenuItem selectAll = new JMenuItem("Select All");
		selectAll.setAccelerator(KeyStroke.getKeyStroke('A',Event.CTRL_MASK ));
		selectAll.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						if (TextAutomatonFrame.getFrame()!=null && 
							    TextAutomatonFrame.getFrame().isSelected()) {
								TextAutomatonFrame.getFrame().graphicalZone.selectAllBoxes();
								TextAutomatonFrame.getFrame().graphicalZone.repaint();
							}
						else if (currentFrame!=null) {
							currentFrame.graphicalZone.selectAllBoxes();
							currentFrame.graphicalZone.repaint();
						} 
					}
				});
		}});
		selectAll.setEnabled(true);
		edit.add(cut);
		edit.add(copy);
		edit.add(paste);
		edit.add(selectAll);
		return edit;
	}


	public JMenu buildWindowsMenu() {
		JMenu windows = new JMenu("Windows");
		JMenuItem tile = new JMenuItem("Tile");
		tile.setEnabled(true);
		tile.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						tileFrames();
					}
				});
		}});
		JMenuItem cascade = new JMenuItem("Cascade");
		cascade.setEnabled(true);
		cascade.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						cascadeFrames();
					}
				});
		}});
		JMenuItem arrangeIcons = new JMenuItem("Arrange Icons");
		arrangeIcons.setEnabled(true);
		arrangeIcons.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						arrangeIcons();
					}
				});
		}});
		windows.add(tile);
		windows.add(cascade);
		windows.add(arrangeIcons);
		return windows;
	}


	public JMenu buildInfoMenu() {
		JMenu info = new JMenu("Info");
		JMenuItem aboutUnitex = new JMenuItem("About Unitex...");
		aboutUnitex.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable(){
					public void run(){
						AboutUnitexFrame.showFrame();
					}
				});
		}});
		aboutUnitex.setEnabled(true);
		JMenuItem references = new JMenuItem("References");
		references.setEnabled(false);
		JMenu helpOnCommands = new JMenu("Help on commands");
			JMenuItem Asc2Uni=new JMenuItem("Asc2Uni");
			Asc2Uni.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new ProcessInfoFrame("\""+Config.getApplicationDir()+"Asc2Uni\"",false);
			}});
			helpOnCommands.add(Asc2Uni);
			JMenuItem CheckDic=new JMenuItem("CheckDic");
			CheckDic.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new ProcessInfoFrame("\""+Config.getApplicationDir()+"CheckDic\"",false);
			}});
			helpOnCommands.add(CheckDic);
			JMenuItem Compress=new JMenuItem("Compress");
			Compress.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new ProcessInfoFrame("\""+Config.getApplicationDir()+"Compress\"",false);
			}});
			helpOnCommands.add(Compress);
			JMenuItem Concord=new JMenuItem("Concord");
			Concord.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new ProcessInfoFrame("\""+Config.getApplicationDir()+"Concord\"",false);
			}});
			helpOnCommands.add(Concord);
			JMenuItem Dico=new JMenuItem("Dico");
			Dico.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new ProcessInfoFrame("\""+Config.getApplicationDir()+"Dico\"",false);
			}});
			helpOnCommands.add(Dico);
			JMenuItem Extract=new JMenuItem("Extract");
			Extract.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new ProcessInfoFrame("\""+Config.getApplicationDir()+"Extract\"",false);
			}});
			helpOnCommands.add(Extract);
			JMenuItem Fst2Grf=new JMenuItem("Fst2Grf");
			Fst2Grf.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new ProcessInfoFrame("\""+Config.getApplicationDir()+"Fst2Grf\"",false);
			}});
			helpOnCommands.add(Fst2Grf);
			JMenuItem Fst2Txt=new JMenuItem("Fst2Txt");
			Fst2Txt.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new ProcessInfoFrame("\""+Config.getApplicationDir()+"Fst2Txt\"",false);
			}});
			helpOnCommands.add(Fst2Txt);
			JMenuItem Grf2Fst2=new JMenuItem("Grf2Fst2");
			Grf2Fst2.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new ProcessInfoFrame("\""+Config.getApplicationDir()+"Grf2Fst2\"",false);
			}});
			helpOnCommands.add(Grf2Fst2);
			JMenuItem Inflect=new JMenuItem("Inflect");
			Inflect.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new ProcessInfoFrame("\""+Config.getApplicationDir()+"Inflect\"",false);
			}});
			helpOnCommands.add(Inflect);
			JMenuItem Locate=new JMenuItem("Locate");
			Locate.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new ProcessInfoFrame("\""+Config.getApplicationDir()+"Locate\"",false);
			}});
			helpOnCommands.add(Locate);
			JMenuItem MergeTextAutomaton=new JMenuItem("MergeTextAutomaton");
			MergeTextAutomaton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new ProcessInfoFrame("\""+Config.getApplicationDir()+"MergeTextAutomaton\"",false);
			}});
			helpOnCommands.add(MergeTextAutomaton);
			JMenuItem Normalize=new JMenuItem("Normalize");
			Normalize.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new ProcessInfoFrame("\""+Config.getApplicationDir()+"Normalize\"",false);
			}});
			helpOnCommands.add(Normalize);
			JMenuItem Reg2Grf=new JMenuItem("Reg2Grf");
			Reg2Grf.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new ProcessInfoFrame("\""+Config.getApplicationDir()+"Reg2Grf\"",false);
			}});
			helpOnCommands.add(Reg2Grf);
			JMenuItem SortTxt=new JMenuItem("SortTxt");
			SortTxt.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new ProcessInfoFrame("\""+Config.getApplicationDir()+"SortTxt\"",false);
			}});
			helpOnCommands.add(SortTxt);
			JMenuItem Table2Grf=new JMenuItem("Table2Grf");
			Table2Grf.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new ProcessInfoFrame("\""+Config.getApplicationDir()+"Table2Grf\"",false);
			}});
			helpOnCommands.add(Table2Grf);
			JMenuItem Tokenize=new JMenuItem("Tokenize");
			Tokenize.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new ProcessInfoFrame("\""+Config.getApplicationDir()+"Tokenize\"",false);
			}});
			helpOnCommands.add(Tokenize);
			JMenuItem Txt2Fst2=new JMenuItem("Txt2Fst2");
			Txt2Fst2.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new ProcessInfoFrame("\""+Config.getApplicationDir()+"Txt2Fst2\"",false);
			}});
			helpOnCommands.add(Txt2Fst2);
			helpOnCommands.setEnabled(true);
		JMenuItem preferences = new JMenuItem("Preferences...");
		preferences.setEnabled(true);
		preferences.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						GlobalPreferenceFrame.showFrame();
					}
				});
		}});
		JMenuItem console = new JMenuItem("Console");
		console.setEnabled(true);
		console.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						Console.getFrame().setVisible(!Console.getFrame().isVisible());
					}
				});
		}});
		info.add(aboutUnitex);
		//info.add(references);
		info.add(helpOnCommands);
		info.addSeparator();
		info.add(preferences);
		info.add(console);
		return info;
	}
	

    public void buildContent() {
        desktop = new JDesktopPane();
        setContentPane(desktop);
    }

    public void quit() {
    	Object[] options = {"Yes","No"};
		int n=JOptionPane.showOptionDialog(this,"Do you really want to quit ?"
			,""
			,JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE,null,options,options[0]);
		if (n==0) {
 			closeAll();
 		}
 		else {
 			return;
 		}

        System.exit(0);
    }

    public void newDocument() {
		GraphFrame doc = new GraphFrame(false);
		desktop.add(doc,DOCLAYER);
		try { 
		    doc.setVisible(true);
		    doc.setSelected(true);
		    doc.setIcon(false);
		} catch (java.beans.PropertyVetoException e2) {}
		currentFrame=doc;
		frames.add(doc);
    }



    public void pageSetup() {
        if (printerJob==null) {
            printerJob=PrinterJob.getPrinterJob();
        } 
        if (pageFormat==null) {
            pageFormat=printerJob.defaultPage();
        }
        pageFormat=printerJob.pageDialog(pageFormat);
    }


    public void printFrame(GraphFrame g) {
        if (printerJob==null) {
            printerJob=PrinterJob.getPrinterJob();
        } 
        if (pageFormat==null) {
            pageFormat=printerJob.defaultPage();
        }
  	printerJob.setPrintable(g.graphicalZone,pageFormat);
        if (printerJob.printDialog()) {
            try {
                printerJob.print();
            } catch (PrinterException e) {
                JOptionPane.showMessageDialog(null,"Error while printing graph","Error",JOptionPane.ERROR_MESSAGE);
            }
        }
    }


    public void printTextAutomatonFrame(TextAutomatonFrame g) {
        if (printerJob==null) {
            printerJob=PrinterJob.getPrinterJob();
        } 
        if (pageFormat==null) {
            pageFormat=printerJob.defaultPage();
        }
        if (printerJob.printDialog()) {
            try {
            	printerJob.setPrintable(g.graphicalZone,pageFormat);        
                printerJob.print();
            } catch (PrinterException e) {
                JOptionPane.showMessageDialog(null,"Error while printing sentence graph","Error",JOptionPane.ERROR_MESSAGE);
            }
        }
    }


    public void printAllFrames() {
    	if (frames.isEmpty()) return;
        if (printerJob==null) {
            printerJob=PrinterJob.getPrinterJob();
        } 
        if (pageFormat==null) {
            pageFormat=printerJob.defaultPage();
        }
        int L=frames.size();
        if (!printerJob.printDialog()) return;
        for (int i=0;i<L;i++) {
        	GraphFrame g=(GraphFrame)frames.get(i);
	        printerJob.setPrintable(g.graphicalZone,pageFormat);
    	    try {
               	printerJob.print();
           	} catch (PrinterException e) {
                JOptionPane.showMessageDialog(null,"Error while printing graph","Error",JOptionPane.ERROR_MESSAGE);
			}
        }
    }


	public void openText() {
		JFileChooser chooser=new JFileChooser();
		chooser.addChoosableFileFilter(new PersonalFileFilter("txt","Raw Unicode Texts"));
		chooser.addChoosableFileFilter(new PersonalFileFilter("snt","Unitex Texts"));
		chooser.setDialogType(JFileChooser.OPEN_DIALOG);
		chooser.setCurrentDirectory(new File(Config.getCurrentCorpusDir()));
		int returnVal=chooser.showOpenDialog(this);
		if (returnVal!=JFileChooser.APPROVE_OPTION) {
			// we return if the user has clicked on CANCEL
			return;
		}
		preprocessText.setEnabled(true);
		applyLexicalResources.setEnabled(true);
		locatePattern.setEnabled(true);
		displayLocatedSequences.setEnabled(true);
		constructFst.setEnabled(true);
		close.setEnabled(true);
		Text.loadCorpus(chooser.getSelectedFile().getAbsolutePath());
	}


	public void openGraph() {
		JFileChooser chooser=new JFileChooser();
		chooser.setFileFilter(new PersonalFileFilter("grf","Unicode Graphs"));
		chooser.setDialogType(JFileChooser.OPEN_DIALOG);
		chooser.setCurrentDirectory(new File(Config.getCurrentGraphDir()));
		int returnVal=chooser.showOpenDialog(this);
		if (returnVal!=JFileChooser.APPROVE_OPTION) {
			// we return if the user has clicked on CANCEL
			return;
		}
		String name,path,fullName;
		try {
			fullName=chooser.getSelectedFile().getCanonicalPath();
			if ( (fullName.lastIndexOf('.')==-1) || (fullName.lastIndexOf('.')!=(fullName.length()-4))) {
				fullName=fullName.concat(".grf");
			}
			name=chooser.getSelectedFile().getName();
			if ((name.lastIndexOf('.')==-1) || (name.lastIndexOf('.')!=(name.length()-4))) {
				name=name.concat(".grf");
			}
			int n=fullName.lastIndexOf(File.separatorChar);
			path=fullName.substring(0,n);
			Config.setCurrentGraphDir(path);
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null,"I/O error while reading on hard disk","Error",JOptionPane.ERROR_MESSAGE);
			return;
		}
		loadGraph(fullName,path,name);
	}

	
	public void openLexiconGrammarTable() {
		JFileChooser chooser=new JFileChooser();
		chooser.setFileFilter(new PersonalFileFilter("txt","Unicode Text Tables"));
		chooser.setDialogType(JFileChooser.OPEN_DIALOG);
		chooser.setCurrentDirectory(new File(Config.getCurrentGraphDir()));
		int returnVal=chooser.showOpenDialog(this);
		if (returnVal!=JFileChooser.APPROVE_OPTION) {
			// we return if the user has clicked on CANCEL
			return;
		}
		String name;
		try {
			name=chooser.getSelectedFile().getCanonicalPath();
		} catch (IOException e) {return;}
		File f=new File(name);
		if (!f.exists()) {
			JOptionPane.showMessageDialog(null,"Cannot find "+name,"Error",JOptionPane.ERROR_MESSAGE);
			return;
		}
		if (!f.canRead()) {
			JOptionPane.showMessageDialog(null,"Cannot read "+name,"Error",JOptionPane.ERROR_MESSAGE);
			return;
		}
		try {
			FileInputStream source=new FileInputStream(name);
			char c=GraphIO.read_char(source);
			source.close();
			if (c!=0xFEFF) {
				JOptionPane.showMessageDialog(null,name+" is not a Unicode Little-Endian lexicon grammar table","Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
		} catch (IOException e) {return;};
		LexiconGrammarTableFrame.closeFrame();
		compileLexiconGrammar.setEnabled(true);
		closeLexiconGrammar.setEnabled(true);
		new LexiconGrammarTableFrame(name);
	}
	

	public GraphFrame graphIsAllreadyOpened(String name) {
		if (frames!=null && !frames.isEmpty()) {
			for (int i=0;i<frames.size();i++) {
				GraphFrame f=(GraphFrame)frames.elementAt(i);
				if (name.equals(f.pathname+File.separatorChar+f.filename)) {
					// if the graph has the good name, we return it
					return f;
				}
			}
		}
		return null;
	}
	

	public void loadGraph(String fullName,String path,String name) {
		GraphFrame doc;
		if ((doc=graphIsAllreadyOpened(fullName))!=null) {
			// we verify if the graph is allready opened
			// in this case, we bring it back to front
			try { 
				doc.setVisible(true);
			    	doc.setSelected(true);
		    		doc.setIcon(false);
			} catch (java.beans.PropertyVetoException e2) {}
			return;
		}
		GraphIO g=GraphIO.loadGraph(fullName);
		if (g==null) return;
		doc=new GraphFrame(true);
		doc.graphicalZone.pref=g.pref;
		doc.graphicalZone.pref.antialiasing=Preferences.pref.antialiasing;
		doc.texte.setFont(doc.graphicalZone.pref.input);
		doc.graphicalZone.Width=g.width;
		doc.graphicalZone.Height=g.height;
		doc.graphicalZone.graphBoxes=g.boxes;
		doc.scroll.setPreferredSize(new Dimension(g.width,g.height));
		doc.graphicalZone.setPreferredSize(new Dimension(g.width,g.height));
		doc.filename=name;
		doc.pathname=path;
		doc.setTitle(doc.filename);
		doc.have_a_name=true;
		desktop.add(doc,DOCLAYER);
		try { 
		    doc.setVisible(true);
		    doc.setSelected(true);
		    doc.setIcon(false);
		} catch (java.beans.PropertyVetoException e2) {}
		currentFrame=doc;
		frames.add(doc);
	}
	

	public static String getExtension(File f) {
		String ext=null;
		String s=f.getName();
		int i=s.lastIndexOf('.');
		if (i>0 && i<s.length()-1) {
			ext = s.substring(i+1).toLowerCase();
		}
		return ext;
	}
	

	public boolean saveAsGraph(GraphFrame f) {
		if (f==null) return false;
		GraphIO g=new GraphIO();
		g.boxes=f.graphicalZone.graphBoxes;
		g.pref=f.graphicalZone.pref;
		g.width=f.graphicalZone.Width;
		g.height=f.graphicalZone.Height;
		String s;
		JFileChooser chooser=new JFileChooser();
		chooser.setFileFilter(new PersonalFileFilter("grf","Unicode Graphs"));
		chooser.setDialogType(JFileChooser.SAVE_DIALOG);
		chooser.setCurrentDirectory(new File(Config.getCurrentGraphDir()));
		int returnVal=chooser.showSaveDialog(this);
		if (returnVal!=JFileChooser.APPROVE_OPTION) {
			// we return if the user has clicked on CANCEL
			return false;
		}
		String name,path,ext;
		try {
			s=chooser.getSelectedFile().getCanonicalPath();
			ext=getExtension(chooser.getSelectedFile());
			if (ext==null || !ext.equalsIgnoreCase("grf"))
				s=s.concat(".grf");
			name=chooser.getSelectedFile().getName();
			if (ext==null || !ext.equalsIgnoreCase("grf"))
				name=name.concat(".grf");
			int n=s.lastIndexOf(File.separatorChar);
			path=s.substring(0,n);
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null,"I/O error while writing on hard disk","Error",JOptionPane.ERROR_MESSAGE);
			return false;
		}
		f.filename=name;
		f.pathname=path;
		f.have_a_name=true;
		f.modified=false;
		g.saveGraph(s);
		f.setTitle(f.filename);
		return true;
	}


	public boolean saveGraph(GraphFrame f) {
		if (f==null) return false;
		if (f.have_a_name==false) {
			return saveAsGraph(f);
		}
		GraphIO g=new GraphIO();
		g.boxes=f.graphicalZone.graphBoxes;
		g.pref=f.graphicalZone.pref;
		g.width=f.graphicalZone.Width;
		g.height=f.graphicalZone.Height;
		f.modified=false;
		g.saveGraph(f.pathname+File.separatorChar+f.filename);
		f.setTitle(f.filename);
		return true;
	}


	public void saveAllGraphs() {
		if (frames.isEmpty()) return;
		int L=frames.size();
		for (int i=0;i<L;i++) {
			GraphFrame f=(GraphFrame)frames.get(i);
			saveGraph(f);
		}
	}


	public void closeAll() {
		while (!frames.isEmpty()) {
			GraphFrame f=(GraphFrame)frames.get(0);
			f.doDefaultCloseAction();
		}
	}

	
	public void compileGraph() {
		if (currentFrame==null) return;
		if (currentFrame.modified==true) {
			JOptionPane.showMessageDialog(null,"Save graph before compiling it","Error",JOptionPane.ERROR_MESSAGE); 
			return;
		}
		if (currentFrame.have_a_name==false) {
			JOptionPane.showMessageDialog(null,"Cannot compile a graph with no name","Error",JOptionPane.ERROR_MESSAGE); 
			return;
		}
		String command="\""+Config.getApplicationDir()+"Grf2Fst2\" \""+currentFrame.pathname+File.separatorChar+currentFrame.filename+"\"";
		new ProcessInfoFrame(command,false);
	}



	public void openDELA() {
		JFileChooser chooser=new JFileChooser();
		chooser.setDialogType(JFileChooser.OPEN_DIALOG);
		chooser.setFileFilter(new PersonalFileFilter("dic","Unicode DELA Dictionaries"));
		chooser.setCurrentDirectory(new File(Config.getUserCurrentLanguageDir()+"Dela"+File.separator));
		int returnVal=chooser.showOpenDialog(this);
		if (returnVal!=JFileChooser.APPROVE_OPTION) {
			// we return if the user has clicked on CANCEL
			return;
		}
		try {
			FileInputStream source=new FileInputStream(chooser.getSelectedFile().getAbsolutePath());
			char c=GraphIO.read_char(source);
			source.close();
			if (c!=0xFEFF) {
				JOptionPane.showMessageDialog(null,chooser.getSelectedFile().getAbsolutePath()+" is not a Unicode Little-Endian dictionary","Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
		} catch (IOException e) {return;};
		Config.setCurrentDELA(chooser.getSelectedFile().getAbsolutePath());
		checkDelaFormat.setEnabled(true);
		sortDictionary.setEnabled(true);
		inflect.setEnabled(true);
		compressIntoFST.setEnabled(true);
		closeDela.setEnabled(true);
		DelaFrame.loadDela(Config.getCurrentDELA());
	}


	public void sortDELA() {
		String command;
		command="\""+Config.getApplicationDir()+"SortTxt\" \"";
		command=command+Config.getCurrentDELA()+"\"";
		if (Config.getCurrentLanguage().equals("Thai") ||
		   	Config.getCurrentLanguage().equals("Korean")) {
			command=command+" -thai ";
		} else {
			command=command+" -o \""+Config.getUserCurrentLanguageDir();
			command=command+"Alphabet_sort.txt\"";
		}
		DelaFrame.closeDela();
		new ProcessInfoFrame(command,true,new DelaDo(Config.getCurrentDELA()));
	}



	public void compressDELA() {
		String command;
		command="\""+Config.getApplicationDir()+"Compress\" \"";
		command=command+Config.getCurrentDELA()+"\"";
		new ProcessInfoFrame(command,false,null);
	}

	

	public void closeDELA() {
		DelaFrame.getFrame().setVisible(false);
		CheckResultFrame.close();
		Config.setCurrentDELA(null);
		checkDelaFormat.setEnabled(false);
		sortDictionary.setEnabled(false);
		inflect.setEnabled(false);
		compressIntoFST.setEnabled(false);
		closeDela.setEnabled(false);
	}


	
	public void closeText() {
		preprocessText.setEnabled(false);
		applyLexicalResources.setEnabled(false);
		locatePattern.setEnabled(false);
		displayLocatedSequences.setEnabled(false);
		constructFst.setEnabled(false);
		close.setEnabled(false);
		Text.closeText();
		TokensFrame.closeTokens();
		closeAllConcordanceFrames();
		TextDicFrame.close();
		TextAutomatonFrame.close();
		repaint();
	}
	
	
	public void closeAllConcordanceFrames() {
		Component[] f=desktop.getComponents();
		for (int i=0;i<f.length;i++) {
			try {
				ConcordanceFrame F=(ConcordanceFrame)f[i];
				F.close();
			} catch (ClassCastException e) {}
		}
	}


	public void tileFrames() {
		Component[] f=desktop.getComponents();
		int openFrameCount=0;
		for (int i=0;i<f.length;i++) {
			try {
				JInternalFrame F=(JInternalFrame)f[i];
				if (F.isVisible() && !F.isIcon()) {
					openFrameCount++;
				}
			} catch (ClassCastException e) {}
		}
		if (openFrameCount==0) return;
		Dimension bounds=getContentPane().getSize();
		if (openFrameCount==1) {
			for (int i=0;i<f.length;i++) {
				try {
					JInternalFrame F=(JInternalFrame)f[i];
					if (F.isVisible() && !F.isIcon()) {
						F.setBounds(0,0,bounds.width,bounds.height);
					}
				} catch (ClassCastException e) {}
				return;
			}	
		}
		if (openFrameCount==2) {
			for (int i=0;i<f.length;i++) {
				try {
					JInternalFrame F=(JInternalFrame)f[i];
					if (F.isVisible() && !F.isIcon()) {
						if (openFrameCount==2)
							F.setBounds(0,0,bounds.width,bounds.height/2);
						else F.setBounds(0,bounds.height/2,bounds.width,bounds.height/2);
					}
					openFrameCount--;
				} catch (ClassCastException e) {}
			}	
			return;
		}
		if (openFrameCount==3) {
			for (int i=0;i<f.length;i++) {
				try {
					JInternalFrame F=(JInternalFrame)f[i];
					if (F.isVisible() && !F.isIcon()) {
						if (openFrameCount==3)
							F.setBounds(0,0,bounds.width,bounds.height/3);
						else if (openFrameCount==2) 
							F.setBounds(0,bounds.height/3,bounds.width,bounds.height/3);
						else F.setBounds(0,2*bounds.height/3,bounds.width,bounds.height/3);
					}
					openFrameCount--;
				} catch (ClassCastException e) {}
			}	
			return;
		}
		int premiere_moitie=openFrameCount/2;
		int seconde_moitie=openFrameCount-premiere_moitie;
		openFrameCount=0;
		// drawing frames on the first half of screen
		int i;
		for (i=0;i<f.length && openFrameCount<premiere_moitie;i++) {
			try {
				JInternalFrame F=(JInternalFrame)f[i];
				if (F.isVisible() && !F.isIcon()) {
					F.setBounds(0,openFrameCount*bounds.height/premiere_moitie,bounds.width/2,bounds.height/premiere_moitie);
				}
				openFrameCount++;
			} catch (ClassCastException e) {}
		}	
		// drawing frames on the second half of screen
		openFrameCount=0;
		for (;i<f.length && openFrameCount<seconde_moitie;i++) {
			try {
				JInternalFrame F=(JInternalFrame)f[i];
				if (F.isVisible() && !F.isIcon()) {
					F.setBounds(bounds.width/2,openFrameCount*bounds.height/seconde_moitie,bounds.width/2,bounds.height/seconde_moitie);
				}
				openFrameCount++;
			} catch (ClassCastException e) {}
		}
	}


	public void cascadeFrames() {
		Component[] f=desktop.getComponents();
		int openFrameCount=0;
		final int offset=30;
		for (int i=0;i<f.length;i++) {
			try {
				JInternalFrame F=(JInternalFrame)f[i];
				if (F.isVisible() && !F.isIcon()) {
					openFrameCount++;
					F.setBounds(offset*(openFrameCount%6),offset*(openFrameCount%6),800,600);
					try {
						F.setSelected(true);
					} catch (java.beans.PropertyVetoException e) {}
				}
			} catch (ClassCastException e) {}
		}
	}


	public void arrangeIcons() {
		Component[] f=desktop.getComponents();
		int openFrameCount=0;
		Dimension desktop_bounds=getContentPane().getSize();
		for (int i=0;i<f.length;i++) {
			try {
				JInternalFrame.JDesktopIcon F=(JInternalFrame.JDesktopIcon)f[i];
				if (F.isVisible()) {
					Dimension icon_bounds=F.getSize();
					int X,Y;
					int n_icons_by_line=(desktop_bounds.width)/(icon_bounds.width);
					if (n_icons_by_line==0) n_icons_by_line=1;
					X=openFrameCount%n_icons_by_line;
					Y=openFrameCount/n_icons_by_line;
					F.setBounds(X*icon_bounds.width,desktop_bounds.height-(Y+1)*icon_bounds.height,icon_bounds.width,icon_bounds.height);
					openFrameCount++;
				}
			} catch (ClassCastException e) {}
		}
	}
	
	
	
class DelaDo extends DoAbstract {
	String dela;
	
	public DelaDo(String s) {
		dela=s;
	}
	public void DO() {
		DelaFrame.loadDela(dela);
	}	
}
}


